@@ -1,4 +1,4 @@
-/*
+﻿/*
  * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 
 /**
  * @file blurayparser.cc
- * @brief Bluray disk parser implementation
+ * @brief Blu-ray disk parser implementation
  *
  * @ingroup ffmpegfs
  *
@@ -44,26 +44,157 @@ extern "C" {
 #include <libavutil/rational.h>
 }
 
-static void stream_info(const std::string &path, BLURAY_STREAM_INFO *ss, int *channels, int *sample_rate, int *audio, int *width, int *height, AVRational *framerate, int *interleaved);
-static int parse_find_best_audio_stream();
-static int parse_find_best_video_stream();
+static bool audio_stream_info(const std::string &path, BLURAY_STREAM_INFO *ss, int *channels, int *sample_rate);
+static bool video_stream_info(const std::string &path, BLURAY_STREAM_INFO *ss, int *width, int *height, AVRational *framerate, int *interleaved);
+static int  parse_find_best_audio_stream();
+static int  parse_find_best_video_stream();
 static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, const std::string & path, const struct stat * statbuf, void * buf, fuse_fill_dir_t filler, bool is_main_title, bool full_title, uint32_t title_idx, uint32_t chapter_idx);
-static int parse_bluray(const std::string & path, const struct stat *statbuf, void *buf, fuse_fill_dir_t filler);
+static int  parse_bluray(const std::string & path, const struct stat *statbuf, void *buf, fuse_fill_dir_t filler);
 
 /**
- * @brief Get information about Bluray stream
- * @param[in] path - Path to Bluray disk.
+ * @brief Get information about Blu-ray stream
+ * @param[in] path - Path to Blu-ray disk.
  * @param[in] ss - BLURAY_STREAM_INFO object.
  * @param[out] channels - Number of audio channels in stream.
  * @param[out] sample_rate - Sample rate of stream.
- * @param[out] audio - 1: stream contains audio, 0: no audio
+ * @return Returns true if stream has video, false if not.
+ */
+static bool audio_stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *channels, int *sample_rate)
+{
+    bool audio = false;
+
+    switch (ss->coding_type)
+    {
+    // Video
+    case 0x01:
+    case 0x02:
+    case 0xea:
+    case 0x1b:
+    case 0x24:
+    {
+        break;
+    }
+        // Audio
+    case 0x03:
+    case 0x04:
+    case 0x80:
+    case 0x81:
+    case 0x82:
+    case 0x83:
+    case 0x84:
+    case 0x85:
+    case 0x86:
+    case 0xa1:
+    case 0xa2:
+    {
+        switch (ss->format)
+        {
+        case BLURAY_AUDIO_FORMAT_MONO:
+        {
+            *channels = 1;  // Mono
+            break;
+        }
+        case BLURAY_AUDIO_FORMAT_STEREO:
+        {
+            *channels = 2;  // Stereo
+            break;
+        }
+        case BLURAY_AUDIO_FORMAT_MULTI_CHAN:
+        {
+            *channels = 2;  // Multi Channel
+            break;
+        }
+        case BLURAY_AUDIO_FORMAT_COMBO:
+        {
+            *channels = 2;  // Stereo ac3/dts
+            break;
+        }
+        default:
+        {
+            Logging::error(path, "Unknown number of audio channels %1. Assuming 2 channel/stereo - may be totally wrong.", ss->format);
+            *channels = 2;  // Stereo
+            break;
+        }
+        }
+
+        switch (ss->rate)
+        {
+        case BLURAY_AUDIO_RATE_48:
+        {
+            *sample_rate = 48000;
+            break;
+        }
+        case BLURAY_AUDIO_RATE_96:
+        {
+            *sample_rate = 96000;
+            break;
+        }
+        case BLURAY_AUDIO_RATE_192:
+        {
+            *sample_rate = 192000;
+            break;
+        }
+            // 48 or 96 ac3/dts
+            // 192 mpl/dts-hd
+        case BLURAY_AUDIO_RATE_192_COMBO:
+        {
+            // *sample_rate = "48/192 Khz";
+            break;
+        }
+            // 48 ac3/dts
+            // 96 mpl/dts-hd
+        case BLURAY_AUDIO_RATE_96_COMBO:
+        {
+            // *sample_rate = "48/96 Khz";
+            break;
+        }
+        default:
+        {
+            Logging::error(path, "Unknown audio sample rate %1. Assuming 48 kHz - may be totally wrong.", ss->rate);
+            *sample_rate = 48000;
+            break;
+        }
+        }
+
+        audio = true;
+        break;
+    }
+    case 0x90:
+    case 0x91:
+    {
+        // Language     ss->lang
+        break;
+    }
+    case 0x92:
+    {
+        // Char Code    ss->char_code);
+        // Language     ss->lang);
+        break;
+    }
+    default:
+    {
+        Logging::error(path, "Unrecognised coding type %<%02x>1.", ss->coding_type);
+        break;
+    }
+    }
+
+    return audio;
+}
+
+/**
+ * @brief Get information about Blu-ray stream
+ * @param[in] path - Path to Blu-ray disk.
+ * @param[in] ss - BLURAY_STREAM_INFO object.
  * @param[out] width - Width of video stream.
  * @param[out] height - Height of video stream.
  * @param[out] framerate - Frame rate of video stream.
  * @param[out] interleaved - 1: video stream is interleaved, 0: video stream is not interleaved.
+ * @return Returns true if stream has video, false if not.
  */
-static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *channels, int *sample_rate, int *audio, int *width, int *height, AVRational *framerate, int *interleaved)
+static bool video_stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *width, int *height, AVRational *framerate, int *interleaved)
 {
+    bool video = false;
+
     switch (ss->coding_type)
     {
     // Video
@@ -220,6 +351,7 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
         }
         }
 
+        video = true;
         break;
     }
         // Audio
@@ -235,76 +367,6 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
     case 0xa1:
     case 0xa2:
     {
-        switch (ss->format)
-        {
-        case BLURAY_AUDIO_FORMAT_MONO:
-        {
-            *channels = 1;  // Mono
-            break;
-        }
-        case BLURAY_AUDIO_FORMAT_STEREO:
-        {
-            *channels = 2;  // Stereo
-            break;
-        }
-        case BLURAY_AUDIO_FORMAT_MULTI_CHAN:
-        {
-            *channels = 2;  // Multi Channel
-            break;
-        }
-        case BLURAY_AUDIO_FORMAT_COMBO:
-        {
-            *channels = 2;  // Stereo ac3/dts
-            break;
-        }
-        default:
-        {
-            Logging::error(path, "Unknown number of audio channels %1. Assuming 2 channel/stereo - may be totally wrong.", ss->format);
-            *channels = 2;  // Stereo
-            break;
-        }
-        }
-
-        switch (ss->rate)
-        {
-        case BLURAY_AUDIO_RATE_48:
-        {
-            *sample_rate = 48000;
-            break;
-        }
-        case BLURAY_AUDIO_RATE_96:
-        {
-            *sample_rate = 96000;
-            break;
-        }
-        case BLURAY_AUDIO_RATE_192:
-        {
-            *sample_rate = 192000;
-            break;
-        }
-            // 48 or 96 ac3/dts
-            // 192 mpl/dts-hd
-        case BLURAY_AUDIO_RATE_192_COMBO:
-        {
-            // *sample_rate = "48/192 Khz";
-            break;
-        }
-            // 48 ac3/dts
-            // 96 mpl/dts-hd
-        case BLURAY_AUDIO_RATE_96_COMBO:
-        {
-            // *sample_rate = "48/96 Khz";
-            break;
-        }
-        default:
-        {
-            Logging::error(path, "Unknown audio sample rate %1. Assuming 48 kHz - may be totally wrong.", ss->rate);
-            *sample_rate = 48000;
-            break;
-        }
-        }
-
-        *audio = 1;
         break;
     }
     case 0x90:
@@ -321,10 +383,12 @@ static void stream_info(const std::string & path, BLURAY_STREAM_INFO *ss, int *c
     }
     default:
     {
-        Logging::error(path, "Unrecognised coding type %<%02x>1", ss->coding_type);
+        Logging::error(path, "Unrecognised coding type %<%02x>1.", ss->coding_type);
         break;
     }
     }
+
+    return video;
 }
 
 /**
@@ -352,17 +416,17 @@ static int parse_find_best_video_stream()
 }
 
 /**
- * @brief Create a virtual file entry of a bluray chapter or title.
- * @param[in] bd - Bluray disk clip info.
- * @param[in] ti - Bluray disk title info.
+ * @brief Create a virtual file entry of a Blu-ray chapter or title.
+ * @param[in] bd - Blu-ray disk clip info.
+ * @param[in] ti - Blu-ray disk title info.
  * @param[in] path - Path to check.
  * @param[in] statbuf - File status structure of original file.
  * @param[in, out] buf - The buffer passed to the readdir() operation.
  * @param[in, out] filler - Function to add an entry in a readdir() operation (see https://libfuse.github.io/doxygen/fuse_8h.html#a7dd132de66a5cc2add2a4eff5d435660)
  * @param[in] is_main_title - true if title_idx is the main title
  * @param[in] full_title - If true, create virtual file of all title. If false, include single chapter only.
- * @param[in] title_idx - Zero-based title index on Bluray
- * @param[in] chapter_idx - Zero-based chapter index on Bluray
+ * @param[in] title_idx - Zero-based title index on Blu-ray
+ * @param[in] chapter_idx - Zero-based chapter index on Blu-ray
  * @note buf and filler can be nullptr. In that case the call will run faster, so these parameters should only be passed if to be filled in.
  * @return On error, returns false. On success, returns true.
  */
@@ -379,7 +443,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 
         if (duration < AV_TIME_BASE)
         {
-            Logging::trace(path, "Title %1: skipping empty title", title_idx + 1);
+            Logging::trace(path, "Title %1: skipping empty title.", title_idx + 1);
             return true;
         }
 
@@ -395,7 +459,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 
         if (duration < AV_TIME_BASE)
         {
-            Logging::trace(path, "Title %1 Chapter %2: skipping empty chapter", title_idx + 1, chapter_idx + 1);
+            Logging::trace(path, "Title %1 Chapter %2: skipping empty chapter.", title_idx + 1, chapter_idx + 1);
             return true;
         }
 
@@ -432,7 +496,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
         // break;
     }
 
-    // Bluray is video format anyway
+    // Blu-ray is video format anyway
     virtualfile->m_format_idx           = 0;
     // Mark title/chapter/angle
     virtualfile->m_full_title           = full_title;
@@ -446,18 +510,13 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
         BITRATE video_bit_rate  = 29*1024*1024; // In case the real bitrate cannot be calculated later, assume 20 Mbit video bitrate
         BITRATE audio_bit_rate  = 256*1024;     // In case the real bitrate cannot be calculated later, assume 256 kBit audio bitrate
 
-        int channels            = 0;
-        int sample_rate         = 0;
-        int audio               = 0;
+        bool audio              = false;
 
-        int width               = 0;
-        int height              = 0;
-        AVRational framerate    = { 0, 0 };
         int interleaved         = 0;
 
         if (!bd_select_title(bd, title_idx))
         {
-            Logging::error(path, "Failed to open bluray title %1", title_idx);
+            Logging::error(path, "The Blu-ray title %1 could not be opened.", title_idx);
             errno = EIO;
             return false;
         }
@@ -469,7 +528,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
         if (duration)
         {
             /**
-             * @todo We actually calculate the overall Bluray bitrate here, including all audio
+             * @todo We actually calculate the overall Blu-ray bitrate here, including all audio
              * streams, not just the video bitrate. This should be the video bitrate alone. We
              * should also calculate the audio bitrate for the selected stream.
             */
@@ -479,22 +538,22 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
         // Get details
         if (clip->audio_stream_count)
         {
-            stream_info(path, &clip->audio_streams[parse_find_best_audio_stream()], &channels, &sample_rate, &audio, &width, &height, &framerate, &interleaved);
+            audio_stream_info(path, &clip->audio_streams[parse_find_best_audio_stream()], &virtualfile->m_channels, &virtualfile->m_sample_rate);
         }
         if (clip->video_stream_count)
         {
-            stream_info(path, &clip->video_streams[parse_find_best_video_stream()], &channels, &sample_rate, &audio, &width, &height, &framerate, &interleaved);
+            audio = video_stream_info(path, &clip->video_streams[parse_find_best_video_stream()], &virtualfile->m_width, &virtualfile->m_height, &virtualfile->m_framerate, &interleaved);
         }
 
-        Logging::trace(virtualfile->m_destfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_bit_rate).c_str(), width, height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
-        if (audio > -1)
+        Logging::trace(path, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_bit_rate).c_str(), virtualfile->m_width, virtualfile->m_height, av_q2d(virtualfile->m_framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
+        if (audio)
         {
-            Logging::trace(virtualfile->m_destfile, "Audio %1 channels %2", channels, format_samplerate(sample_rate).c_str());
+            Logging::trace(path, "Audio %1 channels %2", virtualfile->m_channels, format_samplerate(virtualfile->m_sample_rate).c_str());
         }
 
-        transcoder_set_filesize(virtualfile, duration, audio_bit_rate, channels, sample_rate, AV_SAMPLE_FMT_NONE, video_bit_rate, width, height, interleaved, framerate);
+        transcoder_set_filesize(virtualfile, duration, audio_bit_rate, virtualfile->m_channels, virtualfile->m_sample_rate, AV_SAMPLE_FMT_NONE, video_bit_rate, virtualfile->m_width, virtualfile->m_height, interleaved, virtualfile->m_framerate);
 
-        virtualfile->m_video_frame_count    = static_cast<uint32_t>(av_rescale_q(duration, av_get_time_base_q(), av_inv_q(framerate)));
+        virtualfile->m_video_frame_count    = static_cast<uint32_t>(av_rescale_q(duration, av_get_time_base_q(), av_inv_q(virtualfile->m_framerate)));
         virtualfile->m_predicted_size       = static_cast<size_t>(size);
     }
 
@@ -502,7 +561,7 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
 }
 
 /**
- * @brief Parse Bluray directory and get all Bluray titles and chapters as virtual files.
+ * @brief Parse Blu-ray directory and get all Blu-ray titles and chapters as virtual files.
  * @param[in] path - Path to check.
  * @param[in] statbuf - File status structure of original file.
  * @param[in, out] buf - The buffer passed to the readdir() operation.
@@ -521,7 +580,7 @@ static int parse_bluray(const std::string & path, const struct stat * statbuf, v
 
     bd_dir = path.c_str();
 
-    Logging::debug(path, "Parsing Bluray.");
+    Logging::debug(path, "Parsing Blu-ray.");
 
     bd = bd_open(bd_dir, nullptr);
 
@@ -576,9 +635,9 @@ int check_bluray(const std::string & path, void *buf, fuse_fill_dir_t filler)
     {
         if (!check_path(_path))
         {
-            Logging::trace(_path, "Bluray detected.");
+            Logging::trace(_path, "Blu-ray detected.");
             res = parse_bluray(_path, &stbuf, buf, filler);
-            Logging::trace(_path, "Found %1 titles.", res);
+            Logging::trace(_path, "%1 titles were discovered.", res);
         }
         else
         {
