@@ -87,18 +87,18 @@ bool Buffer::open_file(uint32_t segment_no, uint32_t flags)
 
     if (m_ci[index].m_fd != -1)
     {
-        Logging::trace(m_ci[index].m_cachefile, "Cache file already open, no need to open again.");
+        Logging::trace(m_ci[index].m_cachefile, "Cache file is already open.");
         // Already open
         return true;
     }
 
     if (flags & CACHE_FLAG_RW)
     {
-        Logging::debug(m_ci[index].m_cachefile, "Writing cache file.");
+        Logging::debug(m_ci[index].m_cachefile, "Writing to cache file.");
     }
     else
     {
-        Logging::debug(m_ci[index].m_cachefile, "Reading cache file.");
+        Logging::debug(m_ci[index].m_cachefile, "Reading from cache file.");
     }
 
     size_t filesize     = 0;
@@ -135,20 +135,20 @@ bool Buffer::close_file(uint32_t segment_no, uint32_t flags)
 
     if (m_ci[index].m_flags)
     {
-        Logging::trace(m_ci[index].m_cachefile, "Cache file still in use while trying to close. Currently open: %1", m_cur_open);
+        Logging::trace(m_ci[index].m_cachefile, "While attempting to close, the cache file is still in use. Currently open: %1", m_cur_open);
         return true;
     }
 
     if (m_ci[index].m_fd == -1)
     {
         // Already closed
-        Logging::trace(m_ci[index].m_cachefile, "No need to close unopened cache file. Currenly open: %1", m_cur_open);
+        Logging::trace(m_ci[index].m_cachefile, "No need to close the unopened cache file. Currently open: %1", m_cur_open);
         return true;
     }
 
     Logging::trace(m_ci[index].m_cachefile, "Closing cache file.");
 
-    bool success = unmap_file(m_ci[index].m_cachefile, &m_ci[index].m_fd, &m_ci[index].m_buffer, &m_ci[index].m_buffer_watermark);
+    bool success = unmap_file(m_ci[index].m_cachefile, &m_ci[index].m_fd, &m_ci[index].m_buffer, m_ci[index].m_buffer_size, &m_ci[index].m_buffer_watermark);
 
     m_ci[index].m_buffer_pos    = 0;
     m_ci[index].m_buffer_size   = 0;
@@ -188,7 +188,7 @@ bool Buffer::init(bool erase_cache)
             }
             else
             {
-                Logging::error(filename(), "INTERNAL ERROR: Segment count is 0!");
+                Logging::error(filename(), "INTERNAL ERROR: Buffer::init()! Segment count is 0.");
                 errno = EINVAL;
                 throw false;
             }
@@ -214,7 +214,7 @@ bool Buffer::init(bool erase_cache)
 
         if (cachefile == nullptr)
         {
-            Logging::error(m_ci[0].m_cachefile, "Error opening cache file: Out of memory");
+            Logging::error(m_ci[0].m_cachefile, "Error opening the cache file: out of memory.");
             errno = ENOMEM;
             throw false;
         }
@@ -247,7 +247,12 @@ bool Buffer::init(bool erase_cache)
         // Index only required for frame sets and there is only one.
         if (!m_ci[0].m_cachefile_idx.empty())
         {
-            assert(virtualfile()->m_video_frame_count > 0);
+            if (virtualfile()->m_video_frame_count == 0)
+            {
+                errno = EINVAL;
+                Logging::error(m_ci[0].m_cachefile, "INTERNAL ERROR: Buffer::init()! Frame count is zero (%1) %2", errno, strerror(errno));
+                throw false;
+            }
             assert(sizeof(IMAGE_FRAME) == 32);
 
             size_t filesize     = 0;
@@ -283,7 +288,7 @@ bool Buffer::init(bool erase_cache)
     return success;
 }
 
-bool Buffer::set_segment(uint32_t segment_no)
+bool Buffer::set_segment(uint32_t segment_no, size_t size)
 {
     std::lock_guard<std::recursive_mutex> lck (m_mutex);
 
@@ -305,7 +310,8 @@ bool Buffer::set_segment(uint32_t segment_no)
 
     m_cur_ci = &m_ci[segment_no - 1];
 
-    return true;
+    // Reserve enough buffer space for segment to avoid frequent resizes
+    return reserve(size);
 }
 
 uint32_t Buffer::segment_count()
@@ -350,7 +356,7 @@ bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t
         *fd = ::open(filename.c_str(), O_CREAT | O_RDWR | (truncate ? O_TRUNC : 0), static_cast<mode_t>(0644));
         if (*fd == -1)
         {
-            Logging::error(filename, "Error opening cache file: (%1) %2", errno, strerror(errno));
+            Logging::error(filename, "The cache file could not be opened due to an error: (%1) %2", errno, strerror(errno));
             throw false;
         }
 
@@ -395,6 +401,7 @@ bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t
         if (*p == MAP_FAILED)
         {
             Logging::error(filename, "File mapping failed: (%1) %2 (fd = %3)", errno, strerror(errno), *fd);
+            *p = nullptr;
             throw false;
         }
     }
@@ -412,7 +419,7 @@ bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t
     return success;
 }
 
-bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size_t * filesize) const
+bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size_t len, size_t * filesize) const
 {
     bool success = true;
 
@@ -429,9 +436,9 @@ bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size
 
     if (__p != nullptr)
     {
-        if (munmap(__p, __filesize ? __filesize : static_cast<size_t>(sysconf(_SC_PAGESIZE))) == -1) // Make sure we do not unmap a zero size file (spitzs EINVBAL error)
+        if (munmap(__p, len ? len : static_cast<size_t>(sysconf(_SC_PAGESIZE))) == -1) // Make sure we do not unmap a zero size file (spits EINVAL error)
         {
-            Logging::error(filename, "Unmapping cache file failed: (%1) %2 %3", errno, strerror(errno), __filesize);
+            Logging::error(filename, "Unmapping cache file failed: (%1) %2 Size: %3", errno, strerror(errno), len);
             success = false;
         }
     }
@@ -442,7 +449,7 @@ bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size
         {
             if (ftruncate(__fd, static_cast<off_t>(__filesize)) == -1)
             {
-                Logging::error(filename, "Error calling ftruncate() to resize and close the cache file: (%1) %2 (fd = %3)", errno, strerror(errno), __fd);
+                Logging::error(filename, "Error calling ftruncate() to resize and close the cache file: (%1) %2 (fd = %3) Size: %5", errno, strerror(errno), __fd, __filesize);
                 success = false;
             }
             ::close(__fd);
@@ -451,7 +458,7 @@ bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size
         {
             ::close(__fd);
 
-            if (unlink(filename.c_str()))
+            if (unlink(filename.c_str()) && errno != ENOENT)    // Ignore if file does not exist
             {
                 Logging::error(filename, "Error removing the cache file: (%1) %2 (fd = %3)", errno, strerror(errno), __fd);
                 success = false;
@@ -503,7 +510,7 @@ bool Buffer::release(int flags /*= CACHE_CLOSE_NOOPT*/)
     // Remove index for frame sets. There is only one.
     if (!m_ci[0].m_cachefile_idx.empty())
     {
-        if (!unmap_file(m_ci[0].m_cachefile_idx, &m_ci[0].m_fd_idx, &m_ci[0].m_buffer_idx, &m_ci[0].m_buffer_size_idx))
+        if (!unmap_file(m_ci[0].m_cachefile_idx, &m_ci[0].m_fd_idx, &m_ci[0].m_buffer_idx, m_ci[0].m_buffer_size_idx, &m_ci[0].m_buffer_size_idx))
         {
             success = false;
         }
@@ -601,30 +608,35 @@ bool Buffer::reserve(size_t size)
 
     if (m_cur_ci == nullptr || m_cur_ci->m_buffer == nullptr)
     {
-        errno = EBADF;
+        errno = ENOMEM;
+        //Logging::error(m_cur_ci->m_cachefile, "Error in resize(): (%1) %2 (fd = %3)", errno, strerror(errno), m_cur_ci->m_fd);
         return false;
     }
 
-    bool success = true;
-
-    if (!size)
+    if (m_cur_ci->m_buffer_size >= size)
     {
-        size = m_cur_ci->m_buffer_size;
+        // Do not shrink
+        return true;
     }
 
     m_cur_ci->m_buffer = static_cast<uint8_t*>(mremap(m_cur_ci->m_buffer, m_cur_ci->m_buffer_size, size, MREMAP_MAYMOVE));
-    if (m_cur_ci->m_buffer != nullptr)
+    if (m_cur_ci->m_buffer == MAP_FAILED)
     {
-        m_cur_ci->m_buffer_size = size;
+        Logging::error(m_cur_ci->m_cachefile, "Error calling mremap() to resize the file: (%1) %2 (fd = %3) Old size: %4 New: %5", errno, strerror(errno), m_cur_ci->m_fd, m_cur_ci->m_buffer_size, size);
+        m_cur_ci->m_buffer = nullptr;
+        return false;
     }
 
+    // Save size
+    m_cur_ci->m_buffer_size = size;
+
     if (ftruncate(m_cur_ci->m_fd, static_cast<off_t>(m_cur_ci->m_buffer_size)) == -1)
     {
         Logging::error(m_cur_ci->m_cachefile, "Error calling ftruncate() to resize the file: (%1) %2 (fd = %3)", errno, strerror(errno), m_cur_ci->m_fd);
-        success = false;
+        return false;
     }
 
-    return ((m_cur_ci->m_buffer != nullptr) && success);
+    return true;
 }
 
 size_t Buffer::write(const uint8_t* data, size_t length)
@@ -633,7 +645,7 @@ size_t Buffer::write(const uint8_t* data, size_t length)
 
     if (m_cur_ci == nullptr || m_cur_ci->m_buffer == nullptr)
     {
-        errno = EBADF;
+        errno = ENOMEM;
         return 0;
     }
 
@@ -738,7 +750,7 @@ int Buffer::seek(int64_t offset, int whence, uint32_t segment_no)
 
     if (ci == nullptr || ci->m_buffer == nullptr)
     {
-        errno = EBADF;
+        errno = ENOMEM;
         return (EOF);
     }
 
@@ -795,7 +807,6 @@ size_t Buffer::tell(uint32_t segment_no) const
 
     if (ci == nullptr)
     {
-        assert(false);
         errno = EBADF;
         return 0;
     }
@@ -858,7 +869,7 @@ bool Buffer::copy(uint8_t* out_data, size_t offset, size_t bufsize, uint32_t seg
 
     if (ci->m_buffer == nullptr)
     {
-        errno = EBADF;
+        errno = ENOMEM;
         return false;
     }
 
