@@ -79,7 +79,7 @@ typedef struct CACHE_INFO
     size_t          m_predicted_filesize;       /**< @brief Predicted file size */
     size_t          m_encoded_filesize;         /**< @brief Actual file size after encode */
     uint32_t        m_video_frame_count;        /**< @brief Number of frames in video or 0 if not a video */
-    uint32_t        m_segment_count;            /**< @brief Number of video segments for HLS */
+    uint32_t        m_segment_count;            /**< @brief Number of segments for HLS */
     RESULTCODE      m_result;                   /**< @brief Result code: */
     bool            m_error;                    /**< @brief true if encode failed */
     int             m_errno;                    /**< @brief errno if encode failed */
@@ -141,7 +141,7 @@ public:
      *
      * Opens a cache entry and opens the cache file.
      *
-     * @param[in] virtualfile - virtualfile struct of a file.
+     * @param[in] virtualfile - VIRTUALFILE struct of a file.
      * @return On success, returns pointer to a Cache_Entry. On error, returns nullptr.
      */
     Cache_Entry *           open(LPVIRTUALFILE virtualfile);
@@ -227,7 +227,7 @@ protected:
     bool                    delete_info(const std::string & filename, const std::string & desttype);
     /**
      * @brief Create cache entry object for a VIRTUALFILE.
-     * @param[in] virtualfile - virtualfile struct of a file.
+     * @param[in] virtualfile - VIRTUALFILE struct of a file.
      * @param[in] desttype - Destination type (MP4, WEBM etc.).
      * @return On success, returns pointer to a Cache_Entry. On error, returns nullptr.
      */
