@@ -57,7 +57,7 @@ Cache_Entry::Cache_Entry(Cache *owner, LPVIRTUALFILE virtualfile)
 
     clear();
 
-    Logging::trace(filename(), "Created new cache entry.");
+    Logging::trace(filename(), "Created a new cache entry.");
 }
 
 Cache_Entry::~Cache_Entry()
@@ -113,7 +113,7 @@ void Cache_Entry::clear(bool fetch_file_time /*= true*/)
     {
         struct stat sb;
 
-        if (stat(filename().c_str(), &sb) == -1)
+        if (stat(filename(), &sb) == -1)
         {
             m_cache_info.m_file_time    = 0;
             m_cache_info.m_file_size    = 0;
@@ -188,7 +188,7 @@ bool Cache_Entry::open(bool create_cache /*= true*/)
         erase_cache = true;
     }
 
-    Logging::trace(filename(), "Last transcode finished: Result %1 Erase cache: %2.", m_cache_info.m_result, erase_cache);
+    Logging::trace(filename(), "The last transcode was completed. Result %1 Erasing cache: %2.", m_cache_info.m_result, erase_cache);
 
     // Store access time
     update_access(true);
@@ -310,14 +310,19 @@ bool Cache_Entry::decode_timeout() const
     return (((time(nullptr) - m_cache_info.m_access_time) >= params.m_max_inactive_abort) && m_ref_count <= 1);
 }
 
-const std::string & Cache_Entry::filename() const
+const char * Cache_Entry::filename() const
 {
-    return m_cache_info.m_origfile;
+    return (m_virtualfile != nullptr ? m_virtualfile->m_origfile.c_str() : "");
 }
 
-const std::string & Cache_Entry::destname() const
+const char *Cache_Entry::destname() const
 {
-    return m_cache_info.m_destfile;
+    return (m_virtualfile != nullptr ? m_virtualfile->m_destfile.c_str() : "");
+}
+
+const char * Cache_Entry::virtname() const
+{
+    return (m_virtualfile != nullptr ? m_virtualfile->m_virtfile.c_str() : "");
 }
 
 void Cache_Entry::lock()
@@ -351,25 +356,41 @@ bool Cache_Entry::outdated() const
 
     if (m_cache_info.m_audiobitrate != params.m_audiobitrate)
     {
-        Logging::debug(filename(), "Triggering re-transcode: Selected audio bitrate changed from %1 to %2.", m_cache_info.m_audiobitrate, params.m_audiobitrate);
+        if (m_cache_info.m_audiobitrate)
+        {
+            // Report if old rate is known
+            Logging::debug(filename(), "Triggering re-transcode: Selected audio bitrate changed from %1 to %2.", m_cache_info.m_audiobitrate, params.m_audiobitrate);
+        }
         return true;
     }
 
     if (m_cache_info.m_audiosamplerate != params.m_audiosamplerate)
     {
-        Logging::debug(filename(), "Triggering re-transcode: Selected audio samplerate changed from %1 to %2.", m_cache_info.m_audiosamplerate, params.m_audiosamplerate);
+        if (m_cache_info.m_audiosamplerate)
+        {
+            // Report if old rate is known
+            Logging::debug(filename(), "Triggering re-transcode: Selected audio samplerate changed from %1 to %2.", m_cache_info.m_audiosamplerate, params.m_audiosamplerate);
+        }
         return true;
     }
 
     if (m_cache_info.m_videobitrate != params.m_videobitrate)
     {
-        Logging::debug(filename(), "Triggering re-transcode: Selected video bitrate changed from %1 to %2.", m_cache_info.m_audiobitrate, params.m_audiobitrate);
+        if (m_cache_info.m_videobitrate)
+        {
+            // Report if old rate is known
+            Logging::debug(filename(), "Triggering re-transcode: Selected video bitrate changed from %1 to %2.", m_cache_info.m_audiobitrate, params.m_audiobitrate);
+        }
         return true;
     }
 
     if (m_cache_info.m_videowidth != params.m_videowidth || m_cache_info.m_videoheight != params.m_videoheight)
     {
-        Logging::debug(filename(), "Triggering re-transcode: Selected video witdh/height changed.");
+        if (m_cache_info.m_videowidth && m_cache_info.m_videoheight)
+        {
+            // Report if old dimensions is known
+            Logging::debug(filename(), "Triggering re-transcode: Selected video witdh/height changed from %1/%2 to %3/%4.", m_cache_info.m_videowidth, m_cache_info.m_videoheight, params.m_videowidth, params.m_videoheight);
+        }
         return true;
     }
 
@@ -379,7 +400,7 @@ bool Cache_Entry::outdated() const
         return true;
     }
 
-    if (stat(filename().c_str(), &sb) != -1)
+    if (stat(filename(), &sb) != -1)
     {
         // If source file exists, check file date/size
         if (m_cache_info.m_file_time < sb.st_mtime)
