@@ -64,7 +64,7 @@ private:
 public:
     /**
      * @brief operator = declared deleted, should use create to get this object to maintain reference count.
-     * @param other - Assignment object
+     * @param[in] other - Assignment object
      * @return Pointer to this
      */
     Cache_Entry & operator= (Cache_Entry const & other) = delete;
@@ -140,15 +140,20 @@ public:
      */
     bool                    decode_timeout() const;
     /**
-     * @brief Get the name of the source file.
-     * @return Returns the name of the source file.
-     */
-    const std::string &     filename() const;
-    /**
-     * @brief Get the name of the transcoded file.
+     * @brief Return source filename.
      * @return Returns the name of the transcoded file.
      */
-    const std::string &     destname() const;
+    const char *            filename() const;
+    /**
+     * @brief Return destination filename.
+     * @return Returns the name of the transcoded file.
+     */
+    const char *            destname() const;
+    /**
+     * @brief Return virtual filename. Same as destination filename, but with virtual (mount) path..
+     * @return Returns the name of the transcoded file.
+     */
+    const char *            virtname() const;
     /**
      * @brief Update last access time.
      * @param[in] update_database - If true, also persist in SQL database.
