@@ -163,7 +163,7 @@ static int dvd_find_best_audio_stream(const vtsi_mat_t *vtsi_mat, int *best_chan
 /**
  * @brief Get the frame rate of the DVD. Can be 25 fps (PAL) or  29.97 (NTCS).
  * @param[in] ptr - Pointer to frame_u element in dvd_time_t structure.
- * @return On success returns AVRational with frame rate. On error returns frame rate { 0, 0 }.
+ * @return On success, returns AVRational with frame rate. On error, returns frame rate { 0, 0 }.
  */
 static AVRational dvd_frame_rate(const uint8_t * ptr)
 {
@@ -394,9 +394,17 @@ static bool create_dvd_virtualfile(const ifo_handle_t *vts_file, const std::stri
             }
 
             Logging::trace(virtualfile->m_destfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_settings.m_video_bit_rate).c_str(), video_settings.m_width, video_settings.m_height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
+
+            virtualfile->m_width        = video_settings.m_width;
+            virtualfile->m_height       = video_settings.m_height;
+            virtualfile->m_framerate    = framerate;
+
             if (audio_stream > -1)
             {
-                Logging::trace(virtualfile->m_destfile, "Audio %1 Channels %2", audio_settings.m_channels, audio_settings.m_sample_rate);
+                Logging::trace(virtualfile->m_destfile, "Audio Channels %1 Sample Rate %2", audio_settings.m_channels, audio_settings.m_sample_rate);
+
+                virtualfile->m_channels     = audio_settings.m_channels;
+                virtualfile->m_sample_rate  = audio_settings.m_sample_rate;
             }
 
             transcoder_set_filesize(virtualfile, duration, audio_settings.m_audio_bit_rate, audio_settings.m_channels, audio_settings.m_sample_rate, AV_SAMPLE_FMT_NONE, video_bit_rate, video_settings.m_width, video_settings.m_height, interleaved, framerate);
@@ -425,19 +433,19 @@ static int parse_dvd(const std::string & path, const struct stat *statbuf, void
     int titles;
     bool success = true;
 
-    Logging::debug(path, "Parsing DVD.");
+    Logging::debug(path, "Parsing the DVD.");
 
     dvd = DVDOpen(path.c_str());
     if (dvd == nullptr)
     {
-        Logging::error(path, "Couldn't open DVD.");
+        Logging::error(path, "Couldn't open the DVD.");
         return ENOENT;
     }
 
     ifo_file = ifoOpen(dvd, 0);
     if (ifo_file == nullptr)
     {
-        Logging::error(path, "Can't open VMG info for DVD.");
+        Logging::error(path, "Can't open VMG info for the DVD.");
         DVDClose(dvd);
         return -EINVAL;
     }
@@ -556,7 +564,7 @@ int check_dvd(const std::string & path, void *buf, fuse_fill_dir_t filler)
         {
             Logging::trace(_path, "DVD detected.");
             res = parse_dvd(_path, &stbuf, buf, filler);
-            Logging::trace(_path, "Found %1 titles.", res);
+            Logging::trace(_path, "%1 titles were discovered.", res);
         }
         else
         {
