@@ -51,28 +51,7 @@
 #include <string>
 #include <vector>
 #include <regex>
-
-// Support for libavresample has finally been removed from FFmpeg API
-// as of 14.04.2021, see this commit:
-//
-// commit 420cedd49745b284c35d97b936b71ff79b43bdf7
-// Author: Andreas Rheinhardt <andreas.rheinhardt@outlook.com>
-// Date:   Wed Apr 14 04:33:24 2021 +0200
-//
-//     libavresample: Remove deprecated library
-//
-//     Deprecated in c29038f3041a4080342b2e333c1967d136749c0f.
-//     The resample filter based upon this library has been removed as well.
-
-#if !defined(USE_LIBSWRESAMPLE) && !defined(USE_LIBAVRESAMPLE)
-#error "Must have either libswresample (preferred choice for FFMpeg) or libavresample (with libav)."
-#endif
-
-#ifdef USE_LIBSWRESAMPLE
-#define LAVR_DEPRECATE                      1   /**< @brief Prefer libswresample */
-#else
-#define LAVR_DEPRECATE                      0   /**< @brief Prefer libavresample (for Libav libswresample is not available) */
-#endif
+#include <set>
 
 #ifndef PATH_MAX
 #include <linux/limits.h>
@@ -91,6 +70,8 @@ extern "C" {
 }
 #endif
 
+#define INVALID_STREAM  -1              /**< @brief Denote an invalid stream */
+
 #ifndef LIBAVUTIL_VERSION_MICRO
 #error "LIBAVUTIL_VERSION_MICRO not defined. Missing include header?"
 #endif
@@ -164,15 +145,11 @@ const char *get_media_type_string(enum AVMediaType media_type);
 #define BITRATE int                                                                 /**< @brief For FFmpeg bit rate is an int. */
 #endif
 
-// Make access possible over codecpar if available
-#if LAVF_DEP_AVSTREAM_CODEC
-#define CODECPAR(s)     ((s)->codecpar)                                             /**< @brief Map to codecpar */
-#else
-#define CODECPAR(s)     ((s)->codec)                                                /**< @brief Map to codec */
-#endif
-
 #define STRINGIFY(x) #x                                                             /**< @brief Stringification helper for STRINGIFY. Not to be used separately. */
 #define TOSTRING(x) STRINGIFY(x)                                                    /**< @brief Convert a macro argument into a string constant */
+
+#define SAFE_VALUE(p, v, d)     (((p) != nullptr) ? (p)->v : d)                     /**< @brief Access struct/class pointer safely, return default if nullptr */
+
 /**
   * File types
   */
@@ -289,16 +266,17 @@ struct Format_Options
 {
     friend class FFmpegfs_Format;
 
-    typedef std::vector<AVCodecID>  CODEC_VECT;                /**< @brief Vector with valid codec ids for file format */
+    typedef std::vector<AVCodecID>  CODEC_VECT; /**< @brief Vector with valid codec ids for file format */
 
     /**
       * Format options: Audio/video codecs and sample format
       */
     typedef struct _tagFORMAT
     {
-        CODEC_VECT      m_video_codec;      /**< @brief AVCodec used for video encoding */
-        CODEC_VECT      m_audio_codec;      /**< @brief AVCodec used for audio encoding */
-        AVSampleFormat  m_sample_format;    /**< @brief AVSampleFormat for audio encoding, may be AV_SAMPLE_FMT_NONE for "don't care" */
+        CODEC_VECT      m_video_codec;          /**< @brief AVCodec used for video encoding */
+        CODEC_VECT      m_audio_codec;          /**< @brief AVCodec used for audio encoding */
+        CODEC_VECT      m_subtitle_codec;       /**< @brief AVCodec used for subtitle encoding */
+        AVSampleFormat  m_sample_format;        /**< @brief AVSampleFormat for audio encoding, may be AV_SAMPLE_FMT_NONE for "don't care" */
     } FORMAT;
 
     typedef std::map<SAMPLE_FMT, const FORMAT> FORMAT_MAP;   /**< @brief Map of formats. One entry per format derivative. */
@@ -343,7 +321,7 @@ public:
     AVCodecID           video_codec() const;
     /**
      * @brief Check if video codec/file format combination is supported
-     * @param codec_id - Codec ID to check
+     * @param[in] codec_id - Codec ID to check
      * @return Returns true if supported, false if not.
      */
     bool                is_video_codec_supported(AVCodecID codec_id) const;
@@ -360,7 +338,7 @@ public:
     AVCodecID           audio_codec() const;
     /**
      * @brief Check if audio codec/file format combination is supported
-     * @param codec_id - Codec ID to check
+     * @param[in] codec_id - Codec ID to check
      * @return Returns true if supported, false if not.
      */
     bool                is_audio_codec_supported(AVCodecID codec_id) const;
@@ -386,6 +364,13 @@ public:
      */
     std::string         sample_fmt_list() const;
 
+    /**
+     * @brief Get subtitle codec_id
+     * @param[in] codec_id - Input stream codec ID
+     * @return Returns subtitle codec_id that matches the input stream codec ID, or AV_CODEC_ID_NONE of no match.
+     */
+    AVCodecID           subtitle_codec(AVCodecID codec_id) const;
+
 protected:
     /**
      *  @brief Descriptive name of the format.
@@ -448,7 +433,7 @@ public:
     AVCodecID           video_codec() const;
     /**
      * @brief Check if video codec/file format combination is supported
-     * @param codec_id - Codec ID to check
+     * @param[in] codec_id - Codec ID to check
      * @return Returns true if supported, false if not.
      */
     bool                is_video_codec_supported(AVCodecID codec_id) const;
@@ -465,7 +450,7 @@ public:
     AVCodecID           audio_codec() const;
     /**
      * @brief Check if audio codec/file format combination is supported
-     * @param codec_id - Codec ID to check
+     * @param[in] codec_id - Codec ID to check
      * @return Returns true if supported, false if not.
      */
     bool                is_audio_codec_supported(AVCodecID codec_id) const;
@@ -492,7 +477,14 @@ public:
     std::string         sample_fmt_list() const;
 
     /**
-     * @brief Check if this si some sort of multi file format
+     * @brief Get subtitle codec_id
+     * @param[in] codec_id - Input stream codec ID
+     * @return Returns subtitle codec_id that matches the input stream codec ID, or AV_CODEC_ID_NONE of no match.
+     */
+    AVCodecID           subtitle_codec(AVCodecID codec_id) const;
+
+    /**
+     * @brief Check if this is some sort of multi file format
      * (any of the following: is_frameset() or is_hls()).
      * @return Returns true for a multi file format.
      */
@@ -561,10 +553,9 @@ const std::string & remove_ext(std::string *filepath);
 /**
  * @brief Find extension in allowlist, if existing.
  * @param[in] ext - Extension, if found.
- * @param[in] allowlist - Allow list to inspect.
  * @return Returns true if extension was found, false if there was none
  */
-bool                allow_list_ext(const std::string & ext, const std::string & allowlist);
+bool                allow_list_ext(const std::string & ext);
 /**
  * @brief Find extension in filename, if existing.
  * @param[in] ext - Extension, if found.
@@ -572,6 +563,13 @@ bool                allow_list_ext(const std::string & ext, const std::string &
  * @return Returns true if extension was found, false if there was none
  */
 bool                find_ext(std::string * ext, const std::string & filename);
+/**
+ * @brief Check if filename has a certain extension. The check is case sensitive.
+ * @param ext - Extension to check.
+ * @param filename - Filename to check
+ * @return Returns true if extension matches, false if not
+ */
+bool                check_ext(const std::string & ext, const std::string & filename);
 /**
  * @brief Replace extension in filename, taking into account that there might not be an extension already.
  * @param[in] filepath - Filename to replace extension.
@@ -598,16 +596,31 @@ char *              new_strdup(const std::string & str);
  * @return Returns std::string with the error defined by errnum.
  */
 std::string         ffmpeg_geterror(int errnum);
+
 /**
- * @brief Convert a FFmpeg time from time_base into standard AV_TIME_BASE fractional seconds.
+ * @brief Convert a FFmpeg time from in timebase to outtime base.
  *
+ * If out time base is omitted, returns standard AV_TIME_BASE fractional seconds
  * Avoids conversion of AV_NOPTS_VALUE.
  *
- * @param[in] ts - Source time.
- * @param[in] time_base - Time base of source.
+ * @param[in] ts - Time in input timebase.
+ * @param[in] timebase_in - Input timebase.
+ * @param[in] timebase_out - Output timebase, defaults to AV_TIMEBASE if unset.
  * @return Returns converted value, or AV_NOPTS_VALUE if ts is AV_NOPTS_VALUE.
  */
-int64_t             ffmpeg_rescale(int64_t ts, const AVRational & time_base);
+int64_t             ffmpeg_rescale_q(int64_t ts, const AVRational & timebase_in, const AVRational & timebase_out = av_get_time_base_q());
+/**
+ * @brief Convert a FFmpeg time from in timebase to out timebase with rounding.
+ *
+ * If out time base is omitted, returns standard AV_TIME_BASE fractional seconds
+ * Avoids conversion of AV_NOPTS_VALUE.
+ *
+ * @param[in] ts - Time in input timebase.
+ * @param[in] timebase_in - Input timebase.
+ * @param[in] timebase_out - Output timebase, defaults to AV_TIMEBASE if unset.
+ * @return Returns converted value, or AV_NOPTS_VALUE if ts is AV_NOPTS_VALUE.
+ */
+int64_t             ffmpeg_rescale_q_rnd(int64_t ts, const AVRational & timebase_in, const AVRational & timebase_out = av_get_time_base_q());
 /**
  * @brief Format numeric value.
  * @param[in] value - Value to format.
@@ -690,19 +703,35 @@ std::string &       rtrim(std::string &s);
 std::string &       trim(std::string &s);
 /**
  * @brief Same as std::string replace(), but replaces all occurrences.
- * @param[in] str - Source string.
+ * @param[inout] str - Source string.
  * @param[in] from - String to replace.
  * @param[in] to - Replacement string.
  * @return Source string with all occurrences of from replaced with to.
  */
 std::string         replace_all(std::string str, const std::string& from, const std::string& to);
+/**
+ * @brief Same as std::string replace(), but replaces string in-place.
+ * @param[inout] str - Source string.
+ * @param[in] from - String to replace.
+ * @param[in] to - Replacement string.
+ * @return Source string with all occurrences of from replaced with to.
+ */
+std::string         replace_all(std::string * str, const std::string& from, const std::string& to = "");
+/**
+ * @brief Replace start of string from "from" to "to".
+ * @param[inout] str - Source string.
+ * @param[in] from - String to replace.
+ * @param[in] to - Replacement string.
+ * @return True if from has been replaced, false if not.
+ */
+bool                replace_start(std::string *str, const std::string& from, const std::string& to = "");
 /**
  * @brief Format a std::string sprintf-like.
  * @param[in] format - sprintf-like format string.
  * @param[in] args - Arguments.
  * @return Returns the formatted string.
  */
-template<typename ... Args> std::string string_format(const std::string& format, Args ... args);
+template<typename ... Args> std::string strsprintf(const std::string& format, Args ... args);
 /**
  * @brief strcasecmp() equivalent for std::string.
  * @param[in] s1 - std:string #1
@@ -728,7 +757,7 @@ std::string         ffmpeg_libinfo();
 /**
  * @brief Lists all supported codecs and devices.
  * @param[in] device_only - If true lists devices only.
- * @return On success returns 0; on error negative AVERROR.
+ * @return On success, returns 0; on error, a negative AVERROR value.
  */
 int                 show_caps(int device_only);
 /**
@@ -747,7 +776,7 @@ int                 supports_albumart(FILETYPE filetype);
 /**
  * @brief Get the FFmpegfs filetype, desttype must be one of FFmpeg's "official" short names for formats.
  * @param[in] desttype - Destination type (MP4, WEBM etc.).
- * @return On success returns FILETYPE enum; on error returns FILETYPE_UNKNOWN.
+ * @return On success, returns FILETYPE enum; On error, returns FILETYPE_UNKNOWN.
  */
 FILETYPE            get_filetype(const std::string & desttype);
 /**
@@ -760,21 +789,21 @@ std::string         get_filetype_text(FILETYPE filetype);
  * @brief Get the FFmpegfs filetype, desttypelist must be a comma separated list of FFmpeg's "official" short names for formats.
  * Will return the first match. Same as get_filetype, but accepts a comma separated list.
  * @param[in] desttypelist - Destination type list (MP4, WEBM etc.) separated by commas.
- * @return On success returns FILETYPE enum; on error returns FILETYPE_UNKNOWN.
+ * @return On success, returns FILETYPE enum; On error, returns FILETYPE_UNKNOWN.
  */
 FILETYPE            get_filetype_from_list(const std::string & desttypelist);
 /**
  * @brief Print info about an AVStream.
  * @param[in] stream - Stream to print.
- * @return On success returns 0; on error negative AVERROR.
+ * @return On success, returns 0; on error, a negative AVERROR value.
  */
 int                 print_stream_info(const AVStream* stream);
 /**
  * Fill the provided buffer with a string containing a FourCC (four-character
  * code) representation.
  *
- * @param buf - Upon return, filled in with the FourCC representation.
- * @param fourcc - The fourcc to represent
+ * @param[in] buf - Upon return, filled in with the FourCC representation.
+ * @param[in] fourcc - The fourcc to represent
  * @return The buffer in input.
  */
 std::string         fourcc_make_string(std::string * buf, uint32_t fourcc);
@@ -796,7 +825,7 @@ const std::string & expand_path(std::string *tgt, const std::string &src);
 /**
  * @brief Check if path is a mount.
  * @param[in] path - Path to check.
- * @return Returns 1 if path is a mount point; 0 if not. On error returns -1. Check errorno for details.
+ * @return Returns 1 if path is a mount point; 0 if not. On error, returns -1. Check errorno for details.
  */
 int                 is_mount(const std::string & path);
 /**
@@ -843,6 +872,13 @@ std::string         sanitise_filepath(const std::string & filepath);
  */
 std::string         sanitise_filepath(std::string * filepath);
 
+/**
+ * @brief Translate file names from FUSE to the original absolute path.
+ * @param[out] origpath - Upon return, contains the name and path of the original file.
+ * @param[in] path - Filename and relative path of the original file.
+ */
+void                 append_basepath(std::string *origpath, const char* path);
+
 /**
  * @brief Minimal check if codec is an album art.
  * Requires frame_rate to decide whether this is a video stream if codec_id is
@@ -937,7 +973,7 @@ typedef enum ENCODING
  * @param[in] text - Text to be converted
  * @param[in] encoding - Encoding of input text.
  * @return Returns 0 if successful and the converted text,
- * or errno value on error and text is unchanged.
+ * or errno value On error, and text is unchanged.
  */
 int                 to_utf8(std::string & text, const std::string & encoding);
 /**
@@ -995,4 +1031,43 @@ typename std::map<const std::string, const T, comp>::const_iterator search_by_va
     return mapOfWords.cend();
 }
 
+/**
+ * @brief Check if subtitle codec is a text or graphical codec
+ * @param[in] codec_id - Codec to check, must be one of the subtitle codecs.
+ * @return Returns true if codec_id is a text based codec, false if it is bitmap based.
+ */
+bool                is_text_codec(AVCodecID codec_id);
+
+/**
+ * @brief Get first audio stream
+ * @param[in] format_ctx - Format context of file
+ * @param[out] channels - Number of audio channels in stream
+ * @param[out] samplerate - Audio sample rate of stream
+ * @return Returns stream number (value greater or equal zero) or negative errno value
+ */
+int                 get_audio_props(AVFormatContext *format_ctx, int *channels, int *samplerate);
+
+/**
+ * @brief Escape characters that are meaningful to regexp.
+ * @param[in] str - String to escape
+ * @return Returns reference to string.
+ */
+const std::string & regex_escape(std::string *str);
+
+/**
+ * @brief Check if filename should be hidden from output path
+ * @param[in] filename - Name to check
+ * @return Returns true, if filename is blocked, false if not.
+ */
+bool                is_blocked(const std::string & filename);
+
+typedef std::set<std::string, comp> STRINGSET;                  /**< @brief Set of strings, sorted/search case insensitive */
+
+/**
+ * @brief Conbine set of strings into comma separated list.
+ * @param[in] s - std::set object to combine
+ * @return List of strings, separated by commas.
+ */
+std::string         implode(const STRINGSET & s);
+
 #endif
