@@ -68,15 +68,15 @@ extern "C" {
 
 #include "ffmpeg_utils.h"
 
-FFmpegfs_Format     ffmpeg_format[2];               /**< @brief Two FFmpegfs_Format infos, 0: video file, 1: audio file */
-FFMPEGFS_PARAMS     params;                         /**< @brief FFmpegfs command line parameters */
+FFmpegfs_Format ffmpeg_format[2];                       /**< @brief Two FFmpegfs_Format infos, 0: video file, 1: audio file */
+FFMPEGFS_PARAMS params;                                 /**< @brief FFmpegfs command line parameters */
 
 FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     : m_basepath("")                                    // required parameter
     , m_mountpath("")                                   // required parameter
 
-    , m_audio_codec(AV_CODEC_ID_NONE)                    // default: use predefined option
-    , m_video_codec(AV_CODEC_ID_NONE)                    // default: use predefined option
+    , m_audio_codec(AV_CODEC_ID_NONE)                   // default: use predefined option
+    , m_video_codec(AV_CODEC_ID_NONE)                   // default: use predefined option
 
     , m_autocopy(AUTOCOPY_OFF)                          // default: off
     , m_recodesame(RECODESAME_NO)                       // default: off
@@ -89,7 +89,6 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_audiosamplerate(44100)                          // default: 44.1 kHz
     , m_audiochannels(2)                                // default: 2 channels
     , m_sample_fmt(SAMPLE_FMT_DONTCARE)                 // default: use source format
-    , m_extensions("")                                  // default: use list provided by FFmpeg API
 
     // Video
     , m_videobitrate(2*1024*1024)                       // default: 2 MBit
@@ -104,6 +103,10 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_hwaccel_dec_API(HWACCELAPI_NONE)                // default: Use software encoder
     , m_hwaccel_dec_device_type(AV_HWDEVICE_TYPE_NONE)  // default: Use software decoder
     , m_hwaccel_dec_blocked(nullptr)                    // default: No blocked encoders
+
+    // Subtitles
+    , m_no_subtitles(0)                                  // default: enable subtitles
+
     // Album arts
     , m_noalbumarts(0)                                  // default: copy album arts
     // Virtual Script
@@ -132,13 +135,91 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_decoding_errors(0)                              // default: ignore errors
     , m_min_dvd_chapter_duration(1)                     // default: 1 second
     , m_oldnamescheme(0)                                // default: new scheme
+    , m_extensions(new STRINGSET)
+    , m_hide_extensions(new STRINGSET)
     , m_win_smb_fix(1)                                  // default: fix enabled
 {
 }
 
+FFMPEGFS_PARAMS::FFMPEGFS_PARAMS(const FFMPEGFS_PARAMS & other)
+{
+    *this = other;
+}
+
 FFMPEGFS_PARAMS::~FFMPEGFS_PARAMS()
 {
     delete m_hwaccel_dec_blocked;
+    delete m_extensions;
+    delete m_hide_extensions;
+}
+
+FFMPEGFS_PARAMS& FFMPEGFS_PARAMS::operator=(const FFMPEGFS_PARAMS & other) noexcept
+{
+    m_basepath = other.m_basepath;
+    m_mountpath = other.m_mountpath;
+
+    m_audio_codec = other.m_audio_codec;
+    m_video_codec = other.m_video_codec;
+
+    m_autocopy = other.m_autocopy;
+    m_recodesame = other.m_recodesame;
+    m_profile = other.m_profile;
+    m_level = other.m_level;
+
+    m_audiobitrate = other.m_audiobitrate;
+    m_audiosamplerate = other.m_audiosamplerate;
+    m_audiochannels = other.m_audiochannels;
+    m_sample_fmt = other.m_sample_fmt;
+
+    m_videobitrate = other.m_videobitrate;
+    m_videowidth = other.m_videowidth;
+    m_videoheight = other.m_videoheight;
+    m_deinterlace = other.m_deinterlace;
+    m_segment_duration = other.m_segment_duration;
+    m_min_seek_time_diff = other.m_min_seek_time_diff;
+
+    m_hwaccel_enc_API = other.m_hwaccel_enc_API;
+    m_hwaccel_enc_device_type = other.m_hwaccel_enc_device_type;
+    m_hwaccel_enc_device = other.m_hwaccel_enc_device;
+    m_hwaccel_dec_API = other.m_hwaccel_dec_API;
+    m_hwaccel_dec_device_type = other.m_hwaccel_dec_device_type;
+    m_hwaccel_dec_device = other.m_hwaccel_dec_device;
+    m_hwaccel_dec_blocked = other.m_hwaccel_dec_blocked;
+
+    m_no_subtitles = other.m_no_subtitles;
+
+    m_noalbumarts = other.m_noalbumarts;
+
+    m_enablescript = other.m_enablescript;
+    m_scriptfile = other.m_scriptfile;
+    m_scriptsource = other.m_scriptsource;
+
+    m_debug = other.m_debug;
+    m_log_maxlevel = other.m_log_maxlevel;
+    m_log_stderr = other.m_log_stderr;
+    m_log_syslog = other.m_log_syslog;
+    m_logfile = other.m_logfile;
+
+    m_expiry_time = other.m_expiry_time;
+    m_max_inactive_suspend = other.m_max_inactive_suspend;
+    m_max_inactive_abort = other.m_max_inactive_abort;
+    m_prebuffer_size = other.m_prebuffer_size;
+    m_max_cache_size = other.m_max_cache_size;
+    m_min_diskspace = other.m_min_diskspace;
+    m_cachepath = other.m_cachepath;
+    m_disable_cache = other.m_disable_cache;
+    m_cache_maintenance = other.m_cache_maintenance;
+    m_prune_cache = other.m_prune_cache;
+    m_clear_cache = other.m_clear_cache;
+    m_max_threads = other.m_max_threads;
+    m_decoding_errors = other.m_decoding_errors;
+    m_min_dvd_chapter_duration = other.m_min_dvd_chapter_duration;
+    m_oldnamescheme = other.m_oldnamescheme;
+    *m_extensions = *other.m_extensions;
+    *m_hide_extensions = *other.m_hide_extensions;
+    m_win_smb_fix = other.m_win_smb_fix;
+
+    return *this;
 }
 
 bool FFMPEGFS_PARAMS::smart_transcode(void) const
@@ -146,60 +227,9 @@ bool FFMPEGFS_PARAMS::smart_transcode(void) const
     return (ffmpeg_format[1].filetype() != FILETYPE_UNKNOWN && ffmpeg_format[0].filetype() != ffmpeg_format[1].filetype());
 }
 
-int FFMPEGFS_PARAMS::guess_format_idx(const std::string & filepath) const
+const FFmpegfs_Format *FFMPEGFS_PARAMS::current_format(LPCVIRTUALFILE virtualfile) const
 {
-    const AVOutputFormat* oformat = ::av_guess_format(nullptr, filepath.c_str(), nullptr);
-
-    if (oformat != nullptr)
-    {
-        if (!params.smart_transcode())
-        {
-            // Not smart encoding: use first format (video file)
-            return 0;
-        }
-        else
-        {
-            // Smart transcoding
-            if (ffmpeg_format[0].video_codec() != AV_CODEC_ID_NONE && oformat->video_codec != AV_CODEC_ID_NONE && !is_album_art(oformat->video_codec))
-            {
-                // Is a video: use first format (video file)
-                return 0;
-            }
-            else if (ffmpeg_format[1].audio_codec() != AV_CODEC_ID_NONE && oformat->audio_codec != AV_CODEC_ID_NONE)
-            {
-                // For audio only, use second format (audio only file)
-                return 1;
-            }
-        }
-    }
-
-    return 0;
-}
-
-FFmpegfs_Format * FFMPEGFS_PARAMS::current_format(const std::string & filepath)
-{
-    LPCVIRTUALFILE virtualfile = find_file(filepath);
-
-    if (virtualfile != nullptr)
-    {
-        // We know the file
-        return current_format(virtualfile);
-    }
-
-    // Guess the result
-    int format_idx = guess_format_idx(filepath);
-
-    if (format_idx > -1)
-    {
-        return &ffmpeg_format[format_idx];
-    }
-
-    return nullptr;
-}
-
-FFmpegfs_Format *FFMPEGFS_PARAMS::current_format(LPCVIRTUALFILE virtualfile)
-{
-    if (virtualfile->m_format_idx < 0 || virtualfile->m_format_idx > 1)
+    if (virtualfile == nullptr || virtualfile->m_format_idx < 0 || virtualfile->m_format_idx > 1)
     {
         return nullptr;
     }
@@ -245,6 +275,7 @@ enum
     KEY_HWACCEL_DECODER_API,
     KEY_HWACCEL_DECODER_DEVICE,
     KEY_HWACCEL_DECODER_BLOCKED,
+    KEY_HIDE_EXTENSIONS
 };
 
 /**
@@ -309,6 +340,9 @@ static struct fuse_opt ffmpegfs_opts[] =
     FUSE_OPT_KEY("hwaccel_dec_device=%s",           KEY_HWACCEL_DECODER_DEVICE),
     FUSE_OPT_KEY("--hwaccel_dec_blocked=%s",        KEY_HWACCEL_DECODER_BLOCKED),
     FUSE_OPT_KEY("hwaccel_dec_blocked=%s",          KEY_HWACCEL_DECODER_BLOCKED),
+    // Subtitles
+    FFMPEGFS_OPT("--no_subtitles",                  m_no_subtitles, 1),
+    FFMPEGFS_OPT("no_subtitles",                    m_no_subtitles, 1),
     // Album arts
     FFMPEGFS_OPT("--noalbumarts",                   m_noalbumarts, 1),
     FFMPEGFS_OPT("noalbumarts",                     m_noalbumarts, 1),
@@ -352,6 +386,8 @@ static struct fuse_opt ffmpegfs_opts[] =
     FFMPEGFS_OPT("min_dvd_chapter_duration=%u",     m_min_dvd_chapter_duration, 0),
     FFMPEGFS_OPT("--oldnamescheme=%u",              m_oldnamescheme, 0),
     FFMPEGFS_OPT("oldnamescheme=%u",                m_oldnamescheme, 0),
+    FUSE_OPT_KEY("--hide_extensions=%u",            KEY_HIDE_EXTENSIONS),
+    FUSE_OPT_KEY("hide_extensions=%u",              KEY_HIDE_EXTENSIONS),
     // Experimental
     FFMPEGFS_OPT("--win_smb_fix=%u",                m_win_smb_fix, 1),
     FFMPEGFS_OPT("win_smb_fix=%u",                  m_win_smb_fix, 1),
@@ -407,13 +443,11 @@ static const AUDIOCODEC_MAP audiocodec_map =
     { "MP3",            AV_CODEC_ID_MP3 },          // TS, MP4, MOV, MKV
     { "OPUS",           AV_CODEC_ID_OPUS },         // webm
 
-    // TODO: More to come..
-
-    { "VORBIS",         AV_CODEC_ID_VORBIS },       // webm TODO: Sound skips
-    { "DTS",            AV_CODEC_ID_DTS },          // TODO: invalid argument?
-    { "PCM16",          AV_CODEC_ID_PCM_S16LE },    // TODO: is this useable?
-    { "PCM24",          AV_CODEC_ID_PCM_S24LE },    // TODO: is this useable?
-    { "PCM32",          AV_CODEC_ID_PCM_S32LE },    // TODO: is this useable?
+    { "VORBIS",         AV_CODEC_ID_VORBIS },       /** @todo webm: sound skips */
+    { "DTS",            AV_CODEC_ID_DTS },          /** @todo invalid argument? */
+    { "PCM16",          AV_CODEC_ID_PCM_S16LE },    /** @todo is this useable? */
+    { "PCM24",          AV_CODEC_ID_PCM_S24LE },    /** @todo is this useable? */
+    { "PCM32",          AV_CODEC_ID_PCM_S32LE },    /** @todo is this useable? */
 };
 
 /**
@@ -428,9 +462,7 @@ static const VIDEOCODEC_MAP videocodec_map =
     { "VP8",            AV_CODEC_ID_VP8 },          // WebM
     { "VP9",            AV_CODEC_ID_VP9 },          // WebM
 
-    // TODO: More to come..
-
-    //{ "AV1",            AV_CODEC_ID_AV1 },          // WebM   TODO: Ends with "Could not write video frame (error 'Invalid data found when processing input')."
+    //{ "AV1",            AV_CODEC_ID_AV1 },          /** @todo WebM ends with "Could not write video frame (error 'Invalid data found when processing input')." */
 };
 
 /**
@@ -591,6 +623,8 @@ static int          get_codec(const std::string & codec, AVCodecID *codec_id);
 static int          get_hwaccel_dec_blocked(const std::string & arg, HWACCEL_BLOCKED_MAP **hwaccel_dec_blocked);
 static int          get_value(const std::string & arg, int *value);
 static int          get_value(const std::string & arg, std::string *value);
+static int          get_value(const std::string & arg, STRINGSET *value);
+//static int          get_value(const std::string & arg, std::optional<std::string> *value);
 static int          get_value(const std::string & arg, double *value);
 
 static int          ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_args *outargs);
@@ -1566,7 +1600,7 @@ std::string get_hwaccel_API_text(HWACCELAPI hwaccel_API)
 }
 
 /**
- * @brief Get value form command line string.
+ * @brief Get value from command line string.
  * Finds whatever is after the "=" sign.
  * @param[in] arg - Command line option.
  * @param[in] value - Upon return, contains the value after the "=" sign.
@@ -1589,7 +1623,7 @@ static int get_value(const std::string & arg, int *value)
 }
 
 /**
- * @brief Get value form command line string.
+ * @brief Get value from command line string.
  * Finds whatever is after the "=" sign.
  * @param[in] arg - Command line option.
  * @param[in] value - Upon return, contains the value after the "=" sign.
@@ -1612,7 +1646,58 @@ static int get_value(const std::string & arg, std::string *value)
 }
 
 /**
- * @brief Get value form command line string.
+ * @brief Get comma separated values from command line string.
+ * Finds whatever is after the "=" sign.
+ * @param[in] arg - Command line option.
+ * @param[in] value - Upon return, contains a set of the values after the "=" sign.
+ * @return Returns 0 if valid; if invalid returns -1.
+ */
+static int get_value(const std::string & arg, STRINGSET *value)
+{
+    size_t pos = arg.find('=');
+
+    if (pos != std::string::npos)
+    {
+        std::vector<std::string> v = split(arg.substr(pos + 1), ",");
+
+        value->clear();
+        value->insert(v.begin(), v.end());
+
+        return 0;
+    }
+
+    std::fprintf(stderr, "INVALID PARAMETER (%s): Missing argument\n", arg.c_str());
+
+    return -1;
+}
+
+#if 0
+/**
+ * @brief Get value from command line string.
+ * Finds whatever is after the "=" sign.
+ * @param[in] arg - Command line option.
+ * @param[in] value - Upon return, contains the value after the "=" sign.
+ * @return Returns 0 if valid; if invalid returns -1.
+ */
+static int get_value(const std::string & arg, std::optional<std::string> *value)
+{
+    size_t pos = arg.find('=');
+
+    if (pos != std::string::npos)
+    {
+        *value = arg.substr(pos + 1);
+
+        return 0;
+    }
+
+    std::fprintf(stderr, "INVALID PARAMETER (%s): Missing argument\n", arg.c_str());
+
+    return -1;
+}
+#endif
+
+/**
+ * @brief Get value from command line string.
  * Finds whatever is after the "=" sign.
  * @param[in] arg - Command line option.
  * @param[in] value - Upon return, contains the value after the "=" sign.
@@ -1720,7 +1805,7 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
         std::printf("%-20s: %s\n", "DVD Library", "enabled");
 #endif // USE_LIBDVD
 #ifdef USE_LIBBLURAY
-        std::printf("%-20s: %s\n", "Bluray Library", BLURAY_VERSION_STRING);
+        std::printf("%-20s: %s\n", "Blu-ray Library", BLURAY_VERSION_STRING);
 #endif // USE_LIBBLURAY
 
         fuse_opt_add_arg(outargs, "--version");
@@ -1787,7 +1872,7 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
     }
     case KEY_EXTENSIONS:
     {
-        return get_value(arg, &params.m_extensions);
+        return get_value(arg, params.m_extensions);
     }
     case KEY_SCRIPTFILE:
     {
@@ -1880,6 +1965,10 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
 
         return 0;
     }
+    case KEY_HIDE_EXTENSIONS:
+    {
+        return get_value(arg, params.m_hide_extensions);
+    }
     }
 
     return 1;
@@ -1969,6 +2058,8 @@ static void print_params(void)
     Logging::trace(nullptr, "Recode to same fmt: %1", get_recodesame_text(params.m_recodesame).c_str());
     Logging::trace(nullptr, "Profile           : %1", get_profile_text(params.m_profile).c_str());
     Logging::trace(nullptr, "Level             : %1", get_level_text(params.m_level).c_str());
+    Logging::trace(nullptr, "Extra Extensions  : %1", implode(*params.m_extensions).c_str());
+    Logging::trace(nullptr, "Hide Extensions   : %1", implode(*params.m_hide_extensions).c_str());
     Logging::trace(nullptr, "--------- Audio ---------");
     Logging::trace(nullptr, "Codecs            : %1+%2", get_codec_name(ffmpeg_format[0].audio_codec(), true), get_codec_name(ffmpeg_format[1].audio_codec(), true));
     Logging::trace(nullptr, "Bitrate           : %1", format_bitrate(params.m_audiobitrate).c_str());
@@ -1978,10 +2069,6 @@ static void print_params(void)
     {
         Logging::trace(nullptr, "Sample Format     : %1", get_sampleformat_text(params.m_sample_fmt).c_str());
     }
-    if (!params.m_extensions.empty())
-    {
-        Logging::trace(nullptr, "Extensions        : %1", params.m_extensions.c_str());
-    }
     Logging::trace(nullptr, "--------- Video ---------");
     Logging::trace(nullptr, "Codec             : %1", get_codec_name(ffmpeg_format[0].video_codec(), true));
     Logging::trace(nullptr, "Bitrate           : %1", format_bitrate(params.m_videobitrate).c_str());
@@ -1999,6 +2086,8 @@ static void print_params(void)
     Logging::trace(nullptr, "API               : %1", get_hwaccel_API_text(params.m_hwaccel_enc_API).c_str());
     Logging::trace(nullptr, "Frame Buffering   : %1", av_hwdevice_get_type_name(params.m_hwaccel_enc_device_type));
     Logging::trace(nullptr, "Device            : %1", params.m_hwaccel_enc_device.c_str());
+    Logging::trace(nullptr, "--------- Subtitles ---------");
+    Logging::trace(nullptr, "No subtitles      : %1", params.m_no_subtitles ? "yes" : "no");
     Logging::trace(nullptr, "--------- Virtual Script ---------");
     Logging::trace(nullptr, "Create script     : %1", params.m_enablescript ? "yes" : "no");
     Logging::trace(nullptr, "Script file name  : %1", params.m_scriptfile.c_str());
@@ -2055,17 +2144,6 @@ int main(int argc, char *argv[])
 
     init_fuse_ops();
 
-    // Configure FFmpeg
-#if !LAVC_DEP_AV_CODEC_REGISTER
-    // register all the codecs
-    avcodec_register_all();
-#endif // !LAVC_DEP_AV_CODEC_REGISTER
-#if !LAVF_DEP_AV_REGISTER
-    av_register_all();
-#endif // !LAVF_DEP_AV_REGISTER
-#if !LAVC_DEP_AV_FILTER_REGISTER
-    avfilter_register_all();
-#endif // LAVC_DEP_AV_FILTER_REGISTER
     // Redirect FFmpeg logs
     av_log_set_callback(ffmpeg_log);
 
@@ -2094,8 +2172,7 @@ int main(int argc, char *argv[])
     {
         params.m_log_stderr = 1;
         params.m_log_maxlevel = "DEBUG";
-        // av_log_set_level(AV_LOG_DEBUG);
-        av_log_set_level(AV_LOG_INFO);
+        av_log_set_level(AV_LOG_INFO); // Do not use AV_LOG_DEBUG; AV_LOG_INFO is chatty enough
     }
     else
     {
