@@ -42,7 +42,7 @@
  * @section ffmpeg_introduction Introduction to FFmpegfs
  *
  * FFmpegfs: A read-only FUSE filesystem which transcodes various audio and video formats
- * to MP4, WebM and many more on the fly when opened and read.
+ * to MP4, WebM, and many more on the fly when opened and read.
  *
  * FFmpegfs can be download [here](https://github.com/nschlia/ffmpegfs).
  *
@@ -137,7 +137,7 @@ typedef enum HWACCELAPI
 typedef std::multimap<AVCodecID, int> HWACCEL_BLOCKED_MAP;      /**< @brief Map command line option to AVCodecID */
 
 // Format
-extern FFmpegfs_Format     ffmpeg_format[2];                    /**< @brief Two FFmpegfs_Format infos, 0: video file, 1: audio file */
+extern FFmpegfs_Format  ffmpeg_format[2];                       /**< @brief Two FFmpegfs_Format infos, 0: video file, 1: audio file */
 
 /**
  * @brief Global program parameters
@@ -145,101 +145,106 @@ extern FFmpegfs_Format     ffmpeg_format[2];                    /**< @brief Two
 extern struct FFMPEGFS_PARAMS
 {
     FFMPEGFS_PARAMS();
-    ~FFMPEGFS_PARAMS();
+    /**
+     * @brief Make copy from other FFMPEGFS_PARAMS object.
+     * @param[in] other - Reference to source FFmpeg_Frame object.
+     * @return Reference to new FFmpeg_Frame object.
+     */
+    FFMPEGFS_PARAMS(const FFMPEGFS_PARAMS & other);
+    ~FFMPEGFS_PARAMS(); // Do not make virtual, breaks fuse API compatibility!
 
     /**
      * @brief Check for smart transcode mode
      * @return true if smart transcode is active, false if not
      */
     bool                smart_transcode(void) const;
-    /**
-     * @brief Try to guess the format index (audio or video) for a file.
-     * @param[in] filepath - Name of the file, path my be included, but not required.
-     * @return Index 0 or 1
-     */
-    int                 guess_format_idx(const std::string & filepath) const;
-    /**
-     * @brief Get FFmpegfs_Format for the file.
-     * @param[in] filepath - Name of the file, path my be included, but not required.
-     * @return On success, returns pointer to format. On error, returns nullptr.
-     */
-    FFmpegfs_Format *   current_format(const std::string & filepath);
+
     /**
      * @brief Get FFmpegfs_Format for a virtual file.
-     * @param[in] virtualfile - virtualfile struct of a file.
+     * @param[in] virtualfile - VIRTUALFILE struct of a file.
      * @return On success, returns pointer to format. On error, returns nullptr.
      */
-    FFmpegfs_Format *   current_format(LPCVIRTUALFILE virtualfile);
+    const FFmpegfs_Format *current_format(LPCVIRTUALFILE virtualfile) const;
+
+    /**
+     * @brief Make copy from other FFMPEGFS_PARAMS object.
+     * @param[in] other - Reference to source FFmpeg_Frame object.
+     * @return Reference to new FFmpeg_Frame object.
+     */
+    FFMPEGFS_PARAMS& operator=(const FFMPEGFS_PARAMS & other) noexcept;
 
     // Paths
-    std::string         m_basepath;                 /**< @brief Base path: Files from this directory (including all sub directories) will be mapped to m_mountpath. */
-    std::string         m_mountpath;                /**< @brief Mount path: Files from m_mountpath will be mapped to this directory. */
+    std::string             m_basepath;                     /**< @brief Base path: Files from this directory (including all sub directories) will be mapped to m_mountpath. */
+    std::string             m_mountpath;                    /**< @brief Mount path: Files from m_mountpath will be mapped to this directory. */
 
     // Output type
-    AVCodecID           m_audio_codec;              /**< @brief Either AV_CODEC_ID_NONE for default, or a user selected codec */
-    AVCodecID           m_video_codec;              /**< @brief Either AV_CODEC_ID_NONE for default, or a user selected codec */
-    AUTOCOPY            m_autocopy;                 /**< @brief Copy streams if codec matches */
-    RECODESAME          m_recodesame;               /**< @brief Recode to same format options */
-    PROFILE             m_profile;					/**< @brief Target profile: Firefox, MS Edge/IE or other */
-    PRORESLEVEL         m_level;                    /**< @brief Level, currently proxy/hq/lt/HQ (ProRes only) */
+    AVCodecID               m_audio_codec;                  /**< @brief Either AV_CODEC_ID_NONE for default, or a user selected codec */
+    AVCodecID               m_video_codec;                  /**< @brief Either AV_CODEC_ID_NONE for default, or a user selected codec */
+    AUTOCOPY                m_autocopy;                     /**< @brief Copy streams if codec matches */
+    RECODESAME              m_recodesame;                   /**< @brief Recode to same format options */
+    PROFILE                 m_profile;                      /**< @brief Target profile: Firefox, MS Edge/IE or other */
+    PRORESLEVEL             m_level;                        /**< @brief Level, currently proxy/hq/lt/HQ (ProRes only) */
     // Audio
-    BITRATE             m_audiobitrate;             /**< @brief Output audio bit rate (bits per second) */
-    int                 m_audiosamplerate;          /**< @brief Output audio sample rate (in Hz) */
-    int                 m_audiochannels;            /**< @brief Max. number of audio channels */
-    SAMPLE_FMT          m_sample_fmt;               /**< @brief Sample format */
-    std::string         m_extensions;               /**< @brief Comma separated lists of supplementary file extensions */
+    BITRATE                 m_audiobitrate;                 /**< @brief Output audio bit rate (bits per second) */
+    int                     m_audiosamplerate;              /**< @brief Output audio sample rate (in Hz) */
+    int                     m_audiochannels;                /**< @brief Max. number of audio channels */
+    SAMPLE_FMT              m_sample_fmt;                   /**< @brief Sample format */
     // Video
-    BITRATE             m_videobitrate;             /**< @brief Output video bit rate (bits per second) */
-    int                 m_videowidth;               /**< @brief Output video width */
-    int                 m_videoheight;              /**< @brief Output video height */
-    int                 m_deinterlace;              /**< @brief 1: deinterlace video, 0: no deinterlace */
+    BITRATE                 m_videobitrate;                 /**< @brief Output video bit rate (bits per second) */
+    int                     m_videowidth;                   /**< @brief Output video width */
+    int                     m_videoheight;                  /**< @brief Output video height */
+    int                     m_deinterlace;                  /**< @brief 1: deinterlace video, 0: no deinterlace */
     // HLS Options
-    int64_t             m_segment_duration;         /**< @brief Duration of one HLS segment file, in AV_TIME_BASE fractional seconds. */
-    int64_t             m_min_seek_time_diff;       /**< @brief Minimum time diff from current to next requested segment to perform a seek, in AV_TIME_BASE fractional seconds. */
+    int64_t                 m_segment_duration;             /**< @brief Duration of one HLS segment file, in AV_TIME_BASE fractional seconds. */
+    int64_t                 m_min_seek_time_diff;           /**< @brief Minimum time diff from current to next requested segment to perform a seek, in AV_TIME_BASE fractional seconds. */
     // Hardware acceleration
-    HWACCELAPI          m_hwaccel_enc_API;          /**< @brief Encoder API */
-    AVHWDeviceType      m_hwaccel_enc_device_type;  /**< @brief Enable hardware acceleration buffering for encoder */
-    std::string         m_hwaccel_enc_device;       /**< @brief Encoder device. May be AUTO to auto detect or empty */
-    HWACCELAPI          m_hwaccel_dec_API;          /**< @brief Decoder API */
-    AVHWDeviceType      m_hwaccel_dec_device_type;  /**< @brief Enable hardware acceleration buffering for decoder */
-    std::string         m_hwaccel_dec_device;       /**< @brief Decoder device. May be AUTO to auto detect or empty */
-    HWACCEL_BLOCKED_MAP*m_hwaccel_dec_blocked;      /**< @brief List of blocked decoders and optional profiles */
+    HWACCELAPI              m_hwaccel_enc_API;              /**< @brief Encoder API */
+    AVHWDeviceType          m_hwaccel_enc_device_type;      /**< @brief Enable hardware acceleration buffering for encoder */
+    std::string             m_hwaccel_enc_device;           /**< @brief Encoder device. May be AUTO to auto detect or empty */
+    HWACCELAPI              m_hwaccel_dec_API;              /**< @brief Decoder API */
+    AVHWDeviceType          m_hwaccel_dec_device_type;      /**< @brief Enable hardware acceleration buffering for decoder */
+    std::string             m_hwaccel_dec_device;           /**< @brief Decoder device. May be AUTO to auto detect or empty */
+    HWACCEL_BLOCKED_MAP*    m_hwaccel_dec_blocked;          /**< @brief List of blocked decoders and optional profiles */
+    // Subtitles
+    int                     m_no_subtitles;                  /**< @brief 0: allow subtitles, 1: do no transcode subtitles */
     // Album arts
-    int                 m_noalbumarts;              /**< @brief skip album arts */
+    int                     m_noalbumarts;                  /**< @brief Skip album arts */
     // Virtual script
-    int                 m_enablescript;             /**< @brief Enable virtual script */
-    std::string         m_scriptfile;               /**< @brief Script name */
-    std::string         m_scriptsource;             /**< @brief Source script */
+    int                     m_enablescript;                 /**< @brief Enable virtual script */
+    std::string             m_scriptfile;                   /**< @brief Script name */
+    std::string             m_scriptsource;                 /**< @brief Source script */
     // FFmpegfs options
-    int                 m_debug;                    /**< @brief Debug mode (stay in foreground */
-    std::string         m_log_maxlevel;             /**< @brief Max. log level */
-    int                 m_log_stderr;               /**< @brief Log output to standard error */
-    int                 m_log_syslog;               /**< @brief Log output to system log */
-    std::string         m_logfile;                  /**< @brief Output filename if logging to file */
+    int                     m_debug;                        /**< @brief Debug mode (stay in foreground */
+    std::string             m_log_maxlevel;                 /**< @brief Max. log level */
+    int                     m_log_stderr;                   /**< @brief Log output to standard error */
+    int                     m_log_syslog;                   /**< @brief Log output to system log */
+    std::string             m_logfile;                      /**< @brief Output filename if logging to file */
     // Background recoding/caching
-    time_t              m_expiry_time;              /**< @brief Time (seconds) after which an cache entry is deleted */
-    time_t              m_max_inactive_suspend;     /**< @brief Time (seconds) that must elapse without access until transcoding is suspended */
-    time_t              m_max_inactive_abort;       /**< @brief Time (seconds) that must elapse without access until transcoding is aborted */
-    size_t              m_prebuffer_size;           /**< @brief Number of bytes that will be decoded before it can be accessed */
-    size_t              m_max_cache_size;           /**< @brief Max. cache size in MB. When exceeded, oldest entries will be pruned */
-    size_t              m_min_diskspace;            /**< @brief Min. diskspace required for cache */
-    std::string         m_cachepath;                /**< @brief Disk cache path, defaults to $XDG_CACHE_HOME */
-    int                 m_disable_cache;            /**< @brief Disable cache */
-    time_t              m_cache_maintenance;        /**< @brief Prune timer interval */
-    int                 m_prune_cache;              /**< @brief Prune cache immediately */
-    int                 m_clear_cache;              /**< @brief Clear cache on start up */
-    unsigned int        m_max_threads;              /**< @brief Max. number of recoder threads */
+    time_t                  m_expiry_time;                  /**< @brief Time (seconds) after which an cache entry is deleted */
+    time_t                  m_max_inactive_suspend;         /**< @brief Time (seconds) that must elapse without access until transcoding is suspended */
+    time_t                  m_max_inactive_abort;           /**< @brief Time (seconds) that must elapse without access until transcoding is aborted */
+    size_t                  m_prebuffer_size;               /**< @brief Number of bytes that will be decoded before it can be accessed */
+    size_t                  m_max_cache_size;               /**< @brief Max. cache size in MB. When exceeded, oldest entries will be pruned */
+    size_t                  m_min_diskspace;                /**< @brief Min. diskspace required for cache */
+    std::string             m_cachepath;                    /**< @brief Disk cache path, defaults to $XDG_CACHE_HOME */
+    int                     m_disable_cache;                /**< @brief Disable cache */
+    time_t                  m_cache_maintenance;            /**< @brief Prune timer interval */
+    int                     m_prune_cache;                  /**< @brief Prune cache immediately */
+    int                     m_clear_cache;                  /**< @brief Clear cache on start up */
+    unsigned int            m_max_threads;                  /**< @brief Max. number of recoder threads */
     // Miscellanous options
-    int                 m_decoding_errors;          /**< @brief Break transcoding on decoding error */
-    int                 m_min_dvd_chapter_duration; /**< @brief Min. DVD chapter duration. Shorter chapters will be ignored. */
-    int                 m_oldnamescheme;            /**< @brief Use old output name scheme, can create duplicate filenames */
+    int                     m_decoding_errors;              /**< @brief Break transcoding on decoding error */
+    int                     m_min_dvd_chapter_duration;     /**< @brief Min. DVD chapter duration. Shorter chapters will be ignored. */
+    int                     m_oldnamescheme;                /**< @brief Use old output name scheme, can create duplicate filenames */
+    STRINGSET*              m_extensions;                   /**< @brief List of supplementary file extensions. Must be a pointer as the fuse API cannot handle advanced c++ objects. */
+    STRINGSET*              m_hide_extensions;              /**< @brief Set of extensions to block/hide. Must be a pointer as the fuse API cannot handle advanced c++ objects. */
     // Experimental options
-    int                 m_win_smb_fix;              /**< @brief Experimental Windows fix for access to EOF at file open */
-} params;                                           /**< @brief Command line parameters */
+    int                     m_win_smb_fix;                  /**< @brief Experimental Windows fix for access to EOF at file open */
+} params;                                                   /**< @brief Command line parameters */
 
 class Cache_Entry;
 
-extern bool             docker_client;              /**< @brief True if running inside a Docker container */
+extern bool             docker_client;                      /**< @brief True if running inside a Docker container */
 
 /**
  * @brief Fuse operations struct
@@ -303,10 +308,13 @@ bool            transcoder_cache_clear(void);
 /**
  * @brief Add new virtual file to internal list.
  *
- * For Bluray/DVD/VCD actually no physical input file exists, so virtual file and origfile are the same.
+ * For Blu-ray/DVD/VCD, actually, no physical input file exists, so virtual and
+ * origfile are the same.
+ *
  * The input file is handled by the BlurayIO or VcdIO classes.
- * For cue sheets, the original (huge) input file is used. Start positions are sought, end positions
- * reported as EOF.
+ *
+ * For cue sheets, the original (huge) input file is used. Start positions are
+ * sought; end positions are reported as EOF.
  *
  * @param[in] type - Type of virtual file.
  * @param[in] virtfile - Name of virtual file.
@@ -326,7 +334,7 @@ LPVIRTUALFILE   insert_file(VIRTUALTYPE type, const std::string & virtfile, cons
  */
 LPVIRTUALFILE   insert_file(VIRTUALTYPE type, const std::string &virtfile, const std::string & origfile, const struct stat *stbuf, int flags = VIRTUALFLAG_NONE);
 /**
- * @brief Add new virtual directory to internal list. If the file already exists, it will be updated.
+ * @brief Add new virtual directory to the internal list. If the file already exists, it will be updated.
  * @param[in] type - Type of virtual file.
  * @param[in] virtdir - Name of virtual directory.
  * @param[in] stbuf - stat buffer with file size, time etc.
@@ -341,8 +349,14 @@ LPVIRTUALFILE   insert_dir(VIRTUALTYPE type, const std::string & virtdir, const
  */
 LPVIRTUALFILE   find_file(const std::string &virtfile);
 /**
- * @brief Check if path has already been parsed.
- * Only useful if for DVD, Bluray or VCD where it is guaranteed that all files have been parsed whenever the directory is in the hash.
+ * @brief Look for the file in the cache.
+ * @param[in] origfile - Filename and path of file to find.
+ * @return If found, returns VIRTUALFILE object, if not found returns nullptr.
+ */
+LPVIRTUALFILE   find_file_from_orig(const std::string &origfile);
+/**
+ * @brief Check if the path has already been parsed.
+ * Only useful if for DVD, Blu-ray or VCD where it is guaranteed that all files have been parsed whenever the directory is in the hash.
  * @param[in] path - Path to parse.
  * @return Returns true if path was found; false if not.
  */
@@ -424,9 +438,9 @@ std::string 	get_level_text(PRORESLEVEL level);
 std::string  	get_hwaccel_API_text(HWACCELAPI hwaccel_API);
 
 /**
- * @brief Check if codec_id and optional profile is in block list.
- * @param codec_id - Codec ID of rto check
- * @param profile - Profile to check. Set to FF_PROFILE_UNKOWN to ignore.
+ * @brief Check if codec_id and the optional profile are in the block list.
+ * @param[in] codec_id - Codec ID to check
+ * @param[in] profile - Profile to check. Set to FF_PROFILE_UNKOWN to ignore.
  * @return Returns true if codec is in block list, false if not.
  */
 bool            check_hwaccel_dec_blocked(AVCodecID codec_id, int profile);
@@ -434,7 +448,7 @@ bool            check_hwaccel_dec_blocked(AVCodecID codec_id, int profile);
 /**
  * @brief Wrapper to the Fuse filler function.
  * @param[inout] buf - The buffer passed to the readdir() operation. May be nullptr.
- * @param[in] filler - Function pointer to the Fuse update function.  May be nullptr.
+ * @param[in] filler - Function pointer to the Fuse update function. May be nullptr.
  * @param[in] name - The file name of the directory entry. Do not include the path!
  * @param[in] stbuf - File attributes, can be nullptr.
  * @param[in] off - Offset of the next entry or zero.
@@ -443,11 +457,11 @@ bool            check_hwaccel_dec_blocked(AVCodecID codec_id, int profile);
 int             add_fuse_entry(void *buf, fuse_fill_dir_t filler, const char * name, const struct stat *stbuf, off_t off);
 
 /**
- * @brief Create the ./.. entries for a virtual directory
+ * @brief Make dot and double dot entries for a virtual directory.
  * @param[inout] buf - The buffer passed to the readdir() operation. May be nullptr.
- * @param[in] filler - Function pointer to the Fuse update function.  May be nullptr.
- * @param[in] stbuf - File attributes, can be nullptr.
+ * @param[in] filler - Function pointer to the Fuse update function. May be nullptr.
+ * @param[in] stbuf - File attributes. May be nullptr.
  * @param[in] off - Offset of the next entry or zero.
- * @return 1 if buffer is full, zero otherwise or if buf or filler is nullptr.
+ * @return Returns 1 if the buffer is full, zero otherwise. If buf or filler is nullptr, returns zero.
  */
 int             add_dotdot(void *buf, fuse_fill_dir_t filler, const struct stat *stbuf, off_t off);
