@@ -1,4 +1,4 @@
-/*
+﻿/*
  * Copyright (C) 2017-2022 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
  * @file fileio.h
  * @brief FileIO class
  *
- * This class allows transparent access to files from DVD, Bluray, Video CD or
+ * This class allows transparent access to files from DVD, Blu-ray, Video CD or
  * to regular disk files.
  *
  * @ingroup ffmpegfs
@@ -65,8 +65,9 @@ extern "C" {
 /**
   * @brief Image frame header
   *
-  * This image frame header will always start at a 8K boundary of the cache.
-  * It can be used to find the next image by seeking a 8K block starting with tag.
+  * This image frame header will always start at an 8K boundary of the cache.
+  *
+  * It can be used to find the next image by seeking an 8K block starting with the tag.
   */
 typedef struct IMAGE_FRAME
 {
@@ -95,7 +96,7 @@ typedef enum VIRTUALTYPE
     VIRTUALTYPE_DVD,                                                /**< @brief DVD file */
 #endif // USE_LIBDVD
 #ifdef USE_LIBBLURAY
-    VIRTUALTYPE_BLURAY,                                             /**< @brief Bluray disk file */
+    VIRTUALTYPE_BLURAY,                                             /**< @brief Blu-ray disk file */
 #endif // USE_LIBBLURAY
 
     VIRTUALTYPE_BUFFER,                                             /**< @brief Buffer file */
@@ -124,6 +125,14 @@ typedef struct VIRTUALFILE
         , m_duration(0)
         , m_predicted_size(0)
         , m_video_frame_count(0)
+        , m_has_audio(false)
+        , m_has_video(false)
+        , m_has_subtitle(false)
+        , m_channels(0)
+        , m_sample_rate(0)
+        , m_width(0)
+        , m_height(0)
+        , m_framerate{ 0, 0 }
     {
         memset(&m_st, 0, sizeof(m_st));
     }
@@ -134,8 +143,9 @@ typedef struct VIRTUALFILE
     int                     m_flags;                                /**< @brief One of the VIRTUALFLAG_* flags */
 
     int                     m_format_idx;                           /**< @brief Index into params.format[] array */
-    std::string             m_destfile;                             /**< @brief Name of virtual file */
-    std::string             m_origfile;                             /**< @brief Sanitised original file name */
+    std::string             m_destfile;                             /**< @brief Name and path of destination file */
+    std::string             m_virtfile;                             /**< @brief Name and path of virtual file */
+    std::string             m_origfile;                             /**< @brief Sanitised name and path of original file */
     struct stat             m_st;                                   /**< @brief stat structure with size etc. */
 
     bool                    m_full_title;                           /**< @brief If true, ignore m_chapter_no and provide full track */
@@ -143,11 +153,15 @@ typedef struct VIRTUALFILE
     size_t                  m_predicted_size;                       /**< @brief Use this as the size instead of computing it over and over. */
     uint32_t                m_video_frame_count;                    /**< @brief Number of frames in video or 0 if not a video */
 
+    bool                    m_has_audio;                            /**< @brief True if file has an audio track */
+    bool                    m_has_video;                            /**< @brief True if file has a video track */
+    bool                    m_has_subtitle;                            /**< @brief True if file has a subtitle track */
+
     std::vector<char>       m_file_contents;                        /**< @brief Buffer for virtual files */
 
 #ifdef USE_LIBVCD
-    /** @brief Extra value structure for Video CDs
-     *  @note Only available if compiled with -DUSE_LIBVCD
+    /** @brief Extra value structure for Video CDs.
+     *  @note Only available if compiled with -DUSE_LIBVCD.
      */
     struct VCD_CHAPTER
     {
@@ -164,8 +178,8 @@ typedef struct VIRTUALFILE
     }                       m_vcd;                                  /**< @brief S/VCD track/chapter info */
 #endif //USE_LIBVCD
 #ifdef USE_LIBDVD
-    /** @brief Extra value structure for DVDs
-     *  @note Only available if compiled with -DUSE_LIBDVD
+    /** @brief Extra value structure for DVDs.
+     *  @note Only available if compiled with -DUSE_LIBDVD.
      */
     struct DVD_CHAPTER
     {
@@ -180,8 +194,8 @@ typedef struct VIRTUALFILE
     }                       m_dvd;                                  /**< @brief DVD title/chapter info */
 #endif // USE_LIBDVD
 #ifdef USE_LIBBLURAY
-    /** @brief Extra value structure for Bluray disks
-     *  @note Only available if compiled with -DUSE_LIBBLURAY
+    /** @brief Extra value structure for Blu-ray disks.
+     *  @note Only available if compiled with -DUSE_LIBBLURAY.
      */
     struct BLURAY_CHAPTER
     {
@@ -195,9 +209,9 @@ typedef struct VIRTUALFILE
         uint32_t            m_playlist_no;                          /**< @brief Playlist number (1...n) */
         unsigned            m_chapter_no;                           /**< @brief Chapter number (1...n) */
         unsigned            m_angle_no;                             /**< @brief Selected angle number (1...n) */
-    }                       m_bluray;                               /**< @brief Bluray title/chapter info */
+    }                       m_bluray;                               /**< @brief Blu-ray title/chapter info */
 #endif // USE_LIBBLURAY
-    /** @brief Extra value structure for cue sheets
+    /** @brief Extra value structure for cue sheets.
      */
     struct CUESHEET_TRACK
     {
@@ -209,7 +223,7 @@ typedef struct VIRTUALFILE
             , m_nextfile(nullptr)
         {}
 
-        int                 m_tracktotal;                           /**< @brief Total number of tracks in cue sheet */
+        int                 m_tracktotal;                           /**< @brief Total number of tracks in cue sheet. */
         int                 m_trackno;                              /**< @brief Track number */
         std::string         m_artist;                               /**< @brief Track artist */
         std::string         m_title;                                /**< @brief Track title */
@@ -221,11 +235,20 @@ typedef struct VIRTUALFILE
         int64_t             m_duration;                             /**< @brief Track/chapter duration, in AV_TIME_BASE fractional seconds. */
 
         VIRTUALFILE*        m_nextfile;                             /**< @brief Next (probable) file to be played. Used for cuesheet lists. */
-    }                       m_cuesheet;                             /**< @brief Cue sheet data for track */
+    }                       m_cuesheet_track;                       /**< @brief Cue sheet data for track. */
+    std::string             m_cuesheet;                             /**< @brief Cue sheet file contents for physical file. */
+
+    // These may be filled in for DVD/Blu-ray
+    int                     m_channels;                             /**< @brief Audio channels - Filled in for the DVD/Blu-ray directory. */
+    int                     m_sample_rate;                          /**< @brief Audio sample rate - Filled in for the DVD/Blu-ray directory. */
+
+    int                     m_width;                                /**< @brief Video width - Filled in for the DVD/Blu-ray directory. */
+    int                     m_height;                               /**< @brief Video height - Filled in for the DVD/Blu-ray directory. */
+    AVRational              m_framerate;                            /**< @brief Video frame rate - Filled in for the DVD/Blu-ray directory. */
 
 } VIRTUALFILE;
-typedef VIRTUALFILE const *LPCVIRTUALFILE;                          /**< @brief Pointer to const version of VIRTUALFILE */
-typedef VIRTUALFILE *LPVIRTUALFILE;                                 /**< @brief Pointer version of VIRTUALFILE */
+typedef VIRTUALFILE const *LPCVIRTUALFILE;                          /**< @brief Pointer to const version of VIRTUALFILE. */
+typedef VIRTUALFILE *LPVIRTUALFILE;                                 /**< @brief Pointer version of VIRTUALFILE. */
 
 /** @brief Base class for I/O
  */
@@ -245,14 +268,14 @@ public:
      *
      * Free with delete if no longer required.
      *
-     * @param[in] type - VIRTUALTYPE of new object
-     * @return Upon successful completion, #FileIO of requested type. @n
-     * On error (out of memory), returns a nullptr.
+     * @param[in] type - VIRTUALTYPE of new object.
+     * @return Upon successful completion, #FileIO of the requested type.
+     * On error, (out of memory), it returns a nullptr.
      */
     static FileIO *     alloc(VIRTUALTYPE type);
 
     /**
-     * @brief Get type of the virtual file
+     * @brief Get type of the virtual file.
      * @return Returns the type of the virtual file.
      */
     virtual VIRTUALTYPE type() const = 0;
@@ -261,19 +284,19 @@ public:
      * @return Return the ideal buffer size.
      */
     virtual size_t  	bufsize() const = 0;
-    /** @brief Open a virtual file
-     * @param[in] virtualfile - LPCVIRTUALFILE of file to open
+    /** @brief Open a virtual file.
+     * @param[in] virtualfile - LPCVIRTUALFILE of file to open.
      * @return Upon successful completion, #open() returns 0. @n
-     * On error, an nonzero value is returned and errno is set to indicate the error.
+     * On error, a nonzero value is returned and errno is set to indicate the error.
      */
     virtual int         open(LPVIRTUALFILE virtualfile) = 0;
-    /** @brief Read data from file
-     * @param[out] data - buffer to store read bytes in. Must be large enough to hold up to size bytes.
-     * @param[in] size - number of bytes to read
+    /** @brief Read data from a file.
+     * @param[out] data - A buffer to store read bytes in. It must be large enough to hold up to size bytes.
+     * @param[in] size - The number of bytes to read.
      * @return Upon successful completion, #read() returns the number of bytes read. @n
      * This may be less than size. @n
      * On error, the value 0 is returned and errno is set to indicate the error. @n
-     * If at end of file, 0 may be returned by errno not set. error() will return 0 if at EOF.
+     * If at the end of the file, 0 may be returned but errno is not set. error() will return 0 if at EOF.
      */
     virtual size_t      read(void *data, size_t size) = 0;
     /**
