@@ -1,4 +1,4 @@
-﻿/*
+/*
  * Copyright (C) 2006-2008 David Collett
  * Copyright (C) 2008-2013 K. Henriksson
  * Copyright (C) 2017-2022 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
@@ -45,8 +45,10 @@
 #include <unistd.h>
 #include <atomic>
 
-/** @brief 1 millisecond = 1,000,000 Nanoseconds */
-#define MS  *1000000L
+
+#define MS              *1000000L   /**< @brief 1 millisecond = 1,000,000 Nanoseconds */
+#define GRANULARITY     250         /**< @brief Image frame conversion: ms between checks if a picture frame is avaiable */
+#define FRAME_TIMEOUT   10          /**< @brief Image frame conversion: timoute seconds to wait if a picture frame is avaiable */
 
 /**
   * @brief THREAD_DATA struct to pass data from parent to child thread
@@ -63,7 +65,7 @@ typedef struct THREAD_DATA
 static Cache *cache;                            /**< @brief Global cache manager object */
 static volatile bool thread_exit;               /**< @brief Used for shutdown: if true, exit all thread */
 
-static void transcoder_thread(void *arg);
+static int transcoder_thread(void *arg);
 static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len, uint32_t segment_no);
 static int transcode_finish(Cache_Entry* cache_entry, FFmpeg_Transcoder & transcoder);
 
@@ -97,19 +99,26 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
             {
                 if (fuse_interrupted())
                 {
-                    Logging::info(cache_entry->destname(), "Client has gone away.");
+                    Logging::info(cache_entry->virtname(), "The client has gone away.");
                     throw false;
                 }
 
                 if (thread_exit)
                 {
-                    Logging::warning(cache_entry->destname(), "Received thread exit.");
+                    Logging::warning(cache_entry->virtname(), "Thread exit was received.");
                     throw false;
                 }
 
                 if (!reported)
                 {
-                    Logging::trace(cache_entry->destname(), "Segment no. %1: Cache miss at offset %2 (length %3), remaining %4.", segment_no, offset, len, format_size_ex(end - cache_entry->m_buffer->size(segment_no)).c_str());
+                    if (!segment_no)
+                    {
+                        Logging::trace(cache_entry->virtname(), "Cache miss at offset %1 with length %2.", offset, len);
+                    }
+                    else
+                    {
+                        Logging::trace(cache_entry->virtname(), "Cache miss at offset %1 with length %2 for segment no. %3.", offset, len, segment_no);
+                    }
                     reported = true;
                 }
                 sleep(0);
@@ -117,7 +126,14 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
 
             if (reported)
             {
-                Logging::trace(cache_entry->destname(), "Segment no. %1: Cache hit at offset %2 (length %3), remaining %4.", segment_no, offset, len, format_size_ex(end - cache_entry->m_buffer->size(segment_no)).c_str());
+                if (!segment_no)
+                {
+                    Logging::trace(cache_entry->virtname(), "Cache hit at offset %1 with length %2.", offset, len);
+                }
+                else
+                {
+                    Logging::trace(cache_entry->virtname(), "Cache hit at offset %1 with length %2 for segment no. %3.", offset, len, segment_no);
+                }
             }
             success = !cache_entry->m_cache_info.m_error;
         }
@@ -134,7 +150,7 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
  * @brief Close the input file and free everything but the initial buffer.
  * @param[in] cache_entry - corresponding cache entry
  * @param[in] transcoder - Current FFmpeg_Transcoder object.
- * @return On success returns 0; on error negative AVERROR.
+ * @return On success, returns 0; on error, a negative AVERROR value.
  */
 static int transcode_finish(Cache_Entry* cache_entry, FFmpeg_Transcoder &transcoder)
 {
@@ -154,16 +170,16 @@ static int transcode_finish(Cache_Entry* cache_entry, FFmpeg_Transcoder &transco
     cache_entry->m_cache_info.m_errno               = 0;
     cache_entry->m_cache_info.m_averror             = 0;
 
-    Logging::debug(transcoder.destname(), "Finishing file.");
+    Logging::debug(transcoder.virtname(), "Finishing file.");
 
     if (!cache_entry->m_buffer->reserve(cache_entry->m_cache_info.m_encoded_filesize))
     {
-        Logging::debug(transcoder.destname(), "Unable to truncate buffer.");
+        Logging::debug(transcoder.virtname(), "Unable to truncate the buffer.");
     }
 
     if (!transcoder.is_multiformat())
     {
-        Logging::debug(transcoder.destname(), "Predicted size: %1 Final: %2 Diff: %3 (%4%).",
+        Logging::debug(transcoder.virtname(), "Predicted size: %1 Final: %2 Diff: %3 (%4%).",
                        format_size_ex(cache_entry->m_cache_info.m_predicted_filesize).c_str(),
                        format_size_ex(cache_entry->m_cache_info.m_encoded_filesize).c_str(),
                        format_result_size_ex(cache_entry->m_cache_info.m_encoded_filesize, cache_entry->m_cache_info.m_predicted_filesize).c_str(),
@@ -213,12 +229,12 @@ bool transcoder_init(void)
 {
     if (cache == nullptr)
     {
-        Logging::debug(nullptr, "Creating media file cache.");
+        Logging::debug(nullptr, "Creating new media file cache.");
         cache = new(std::nothrow) Cache;
         if (cache == nullptr)
         {
-            Logging::error(nullptr, "Unable to create media file cache. Out of memory.");
-            std::fprintf(stderr, "ERROR: Creating media file cache. Out of memory.\n");
+            Logging::error(nullptr, "Unable to create new media file cache. Out of memory.");
+            std::fprintf(stderr, "ERROR: Creating new media file cache. Out of memory.\n");
             return false;
         }
 
@@ -279,10 +295,10 @@ bool transcoder_set_filesize(LPVIRTUALFILE virtualfile, int64_t duration, BITRAT
         return false;
     }
 
-    FFmpegfs_Format *current_format = params.current_format(virtualfile);
+    const FFmpegfs_Format *current_format = params.current_format(virtualfile);
     if (current_format == nullptr)
     {
-        Logging::error(cache_entry->destname(), "Internal error getting file size.");
+        Logging::error(cache_entry->virtname(), "Internal error getting file size.");
         return false;
     }
 
@@ -290,22 +306,22 @@ bool transcoder_set_filesize(LPVIRTUALFILE virtualfile, int64_t duration, BITRAT
 
     if (!FFmpeg_Transcoder::audio_size(&filesize, current_format->audio_codec(), audio_bit_rate, duration, channels, sample_rate, sample_format))
     {
-        Logging::warning(cache_entry->destname(), "Unsupported audio codec '%1' for format %2.", get_codec_name(current_format->audio_codec(), 0), current_format->desttype().c_str());
+        Logging::warning(cache_entry->virtname(), "Unsupported audio codec '%1' for format %2.", get_codec_name(current_format->audio_codec(), 0), current_format->desttype().c_str());
     }
 
     if (!FFmpeg_Transcoder::video_size(&filesize, current_format->video_codec(), video_bit_rate, duration, width, height, interleaved, framerate))
     {
-        Logging::warning(cache_entry->destname(), "Unsupported video codec '%1' for format %2.", get_codec_name(current_format->video_codec(), 0), current_format->desttype().c_str());
+        Logging::warning(cache_entry->virtname(), "Unsupported video codec '%1' for format %2.", get_codec_name(current_format->video_codec(), 0), current_format->desttype().c_str());
     }
 
     if (!FFmpeg_Transcoder::total_overhead(&filesize, current_format->filetype()))
     {
-        Logging::warning(cache_entry->destname(), "Unsupported file type '%1' for format %2.", get_filetype_text(current_format->filetype()).c_str(), current_format->desttype().c_str());
+        Logging::warning(cache_entry->virtname(), "Unsupported file type '%1' for format %2.", get_filetype_text(current_format->filetype()).c_str(), current_format->desttype().c_str());
     }
 
     cache_entry->m_cache_info.m_predicted_filesize = virtualfile->m_predicted_size = filesize;
 
-    Logging::trace(cache_entry->destname(), "Predicted transcoded size of %1.", format_size_ex(cache_entry->m_cache_info.m_predicted_filesize).c_str());
+    Logging::trace(cache_entry->virtname(), "Predicted transcoded size of %1.", format_size_ex(cache_entry->m_cache_info.m_predicted_filesize).c_str());
 
     return true;
 }
@@ -322,6 +338,7 @@ bool transcoder_predict_filesize(LPVIRTUALFILE virtualfile, Cache_Entry* cache_e
             cache_entry->m_cache_info.m_predicted_filesize  = transcoder.predicted_filesize();
             cache_entry->m_cache_info.m_video_frame_count   = transcoder.video_frame_count();
             cache_entry->m_cache_info.m_segment_count       = transcoder.segment_count();
+            cache_entry->m_cache_info.m_duration            = transcoder.duration();
         }
 
         Logging::trace(transcoder.filename(), "Predicted transcoded size of %1.", format_size_ex(transcoder.predicted_filesize()).c_str());
@@ -433,7 +450,7 @@ Cache_Entry* transcoder_new(LPVIRTUALFILE virtualfile, bool begin_transcode)
         }
         else if (begin_transcode)
         {
-            Logging::trace(cache_entry->destname(), "Reading file from cache.");
+            Logging::trace(cache_entry->virtname(), "Reading file from cache.");
         }
 
         cache_entry->unlock();
@@ -456,11 +473,11 @@ bool transcoder_read(Cache_Entry* cache_entry, char* buff, size_t offset, size_t
 
     if (!segment_no)
     {
-        Logging::trace(cache_entry->destname(), "Reading %1 bytes from offset %2.", len, offset);
+        Logging::trace(cache_entry->virtname(), "Reading %1 bytes from offset %2.", len, offset);
     }
     else
     {
-        Logging::trace(cache_entry->destname(), "Reading %1 bytes from offset %2 for segment no. %3.", len, offset, segment_no);
+        Logging::trace(cache_entry->virtname(), "Reading %1 bytes from offset %2 for segment no. %3.", len, offset, segment_no);
     }
 
     // Store access time
@@ -527,7 +544,7 @@ bool transcoder_read(Cache_Entry* cache_entry, char* buff, size_t offset, size_t
                         check_ignore(cache_entry->size(), offset) &&
                         ((offset + len) > (cache_entry->size())))
                 {
-                    Logging::warning(cache_entry->destname(), "Ignoring Windows' groundless access to last 8K boundary of file.");
+                    Logging::warning(cache_entry->virtname(), "Ignoring Windows' groundless access to the last 8K boundary of the file.");
 
                     errno = 0;
                     *bytes_read = 0;  // We've read nothing
@@ -585,7 +602,7 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
 {
     bool success = false;
 
-    Logging::trace(cache_entry->destname(), "Reading %1 bytes from offset %2 for frame no. %3.", len, offset, frame_no);
+    Logging::trace(cache_entry->virtname(), "Reading %1 bytes from offset %2 for frame no. %3.", len, offset, frame_no);
 
     // Store access time
     cache_entry->update_access();
@@ -611,45 +628,40 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
             // Try to read requested frame, stack a seek to if if this fails.
             if (!cache_entry->m_buffer->read_frame(&data, frame_no))
             {
-                Logging::error(nullptr, "SEEK image frame no. %1", frame_no);
                 cache_entry->m_seek_to_no = frame_no;
             }
-            else if (!offset)
-                Logging::error(nullptr, "NOSEEK image frame no. %1", frame_no);
 
-#define GRANULARITY     250 // ms
-#define FRAME_TIMEOUT   10  // seconds
             int retries = FRAME_TIMEOUT * 1000 / GRANULARITY;
             while (!cache_entry->m_buffer->read_frame(&data, frame_no) && !thread_exit)
             {
                 if (errno != EAGAIN)
                 {
-                    Logging::error(cache_entry->destname(), "Reading image frame no. %1: (%2) %3", frame_no, errno, strerror(errno));
+                    Logging::error(cache_entry->virtname(), "Reading image frame no. %1: (%2) %3", frame_no, errno, strerror(errno));
                     throw false;
                 }
 
                 if (errno == EAGAIN && !--retries)
                 {
                     errno = ETIMEDOUT;
-                    Logging::error(cache_entry->destname(), "Timeout reading image frame no. %1: (%2) %3", frame_no, errno, strerror(errno));
+                    Logging::error(cache_entry->virtname(), "Timeout reading image frame no. %1: (%2) %3", frame_no, errno, strerror(errno));
                     throw false;
                 }
 
                 if (fuse_interrupted())
                 {
-                    Logging::info(cache_entry->destname(), "Client has gone away.");
+                    Logging::info(cache_entry->virtname(), "Client has gone away.");
                     throw false;
                 }
 
                 if (thread_exit)
                 {
-                    Logging::warning(cache_entry->destname(), "Received thread exit.");
+                    Logging::warning(cache_entry->virtname(), "Received thread exit.");
                     throw false;
                 }
 
                 if (!reported)
                 {
-                    Logging::trace(cache_entry->destname(), "Cache miss at offset %<%11zu>1 (length %<%6u>2).", offset, len);
+                    Logging::trace(cache_entry->virtname(), "Frame no. %1: Cache miss at offset %<%11zu>2 (length %<%6u>3).", frame_no, offset, len);
                     reported = true;
                 }
 
@@ -659,7 +671,7 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
 
             if (reported)
             {
-                Logging::trace(cache_entry->destname(), "Frame no. %1: Cache hit  at offset %<%11zu>2 (length %<%6u>3).", frame_no, offset, len);
+                Logging::trace(cache_entry->virtname(), "Frame no. %1: Cache hit  at offset %<%11zu>2 (length %<%6u>3).", frame_no, offset, len);
             }
             success = !cache_entry->m_cache_info.m_error;
         }
@@ -668,7 +680,7 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
             success = cache_entry->m_buffer->read_frame(&data, frame_no);
             if (!success)
             {
-                Logging::error(cache_entry->destname(), "Reading image frame no. %1: (%2) %3", frame_no, errno, strerror(errno));
+                Logging::error(cache_entry->virtname(), "Reading image frame no. %1: (%2) %3", frame_no, errno, strerror(errno));
                 throw false;
             }
         }
@@ -753,8 +765,9 @@ bool transcoder_cache_clear(void)
 /**
  * @brief Transcoding thread
  * @param[in] arg - Corresponding Cache_Entry object.
+ * @returns Returns 0 on success; or errno code on error.
  */
-static void transcoder_thread(void *arg)
+static int transcoder_thread(void *arg)
 {
     THREAD_DATA *thread_data = static_cast<THREAD_DATA*>(arg);
     Cache_Entry *cache_entry = static_cast<Cache_Entry *>(thread_data->m_arg);
@@ -809,6 +822,11 @@ static void transcoder_thread(void *arg)
             cache_entry->m_cache_info.m_segment_count   = transcoder.segment_count();
         }
 
+        if (!cache_entry->m_cache_info.m_duration)
+        {
+            cache_entry->m_cache_info.m_duration        = transcoder.duration();
+        }
+
         if (!cache->maintenance(transcoder.predicted_filesize()))
         {
             throw (static_cast<int>(errno));
@@ -834,7 +852,7 @@ static void transcoder_thread(void *arg)
         }
         else
         {
-            Logging::debug(cache_entry->destname(), "Pre-buffering up to %1 bytes.", params.m_prebuffer_size);
+            Logging::debug(cache_entry->virtname(), "Pre-buffering up to %1 bytes.", params.m_prebuffer_size);
         }
 
         while (!cache_entry->is_finished() && !(timeout = cache_entry->decode_timeout()) && !thread_exit)
@@ -892,7 +910,7 @@ static void transcoder_thread(void *arg)
             if (!unlocked && cache_entry->m_buffer->buffer_watermark() > params.m_prebuffer_size)
             {
                 unlocked = true;
-                Logging::debug(cache_entry->destname(), "Pre-buffer limit reached.");
+                Logging::debug(cache_entry->virtname(), "Pre-buffer limit reached.");
                 thread_data->m_lock_guard = true;
                 thread_data->m_cond.notify_all();       // signal that we are running
             }
@@ -906,7 +924,7 @@ static void transcoder_thread(void *arg)
                     thread_data->m_cond.notify_all();  // signal that we are running
                 }
 
-                Logging::info(cache_entry->destname(), "Suspend timeout. Transcoding suspended after %1 seconds inactivity.", params.m_max_inactive_suspend);
+                Logging::info(cache_entry->virtname(), "Suspend timeout. Transcoding suspended after %1 seconds inactivity.", params.m_max_inactive_suspend);
 
                 while (cache_entry->suspend_timeout() && !(timeout = cache_entry->decode_timeout()) && !thread_exit)
                 {
@@ -919,13 +937,13 @@ static void transcoder_thread(void *arg)
                     break;
                 }
 
-                Logging::info(cache_entry->destname(), "Transcoding resumed.");
+                Logging::info(cache_entry->virtname(), "Transcoding resumed.");
             }
         }
 
         if (!unlocked && params.m_prebuffer_size)
         {
-            Logging::debug(cache_entry->destname(), "File transcode complete, releasing buffer early: Size %1.", cache_entry->m_buffer->buffer_watermark());
+            Logging::debug(cache_entry->virtname(), "File transcode complete, releasing buffer early: Size %1.", cache_entry->m_buffer->buffer_watermark());
             thread_data->m_lock_guard = true;
             thread_data->m_cond.notify_all();       // signal that we are running
         }
@@ -966,11 +984,11 @@ static void transcoder_thread(void *arg)
 
             if (timeout)
             {
-                Logging::warning(cache_entry->destname(), "Timeout! Transcoding aborted after %1 seconds inactivity.", params.m_max_inactive_abort);
+                Logging::warning(cache_entry->virtname(), "Timeout! Transcoding aborted after %1 seconds inactivity.", params.m_max_inactive_abort);
             }
             else
             {
-                Logging::info(cache_entry->destname(), "Thread exit! Transcoding aborted.");
+                Logging::info(cache_entry->virtname(), "Thread exit! Transcoding aborted.");
             }
         }
         else
@@ -991,18 +1009,18 @@ static void transcoder_thread(void *arg)
 
         if (success)
         {
-            Logging::info(cache_entry->destname(), "Transcoding completed successfully.");
+            Logging::info(cache_entry->virtname(), "Transcoding completed successfully.");
         }
         else
         {
-            Logging::error(cache_entry->destname(), "Transcoding exited with error.");
+            Logging::error(cache_entry->virtname(), "Transcoding exited with error.");
             if (cache_entry->m_cache_info.m_errno)
             {
-                Logging::error(cache_entry->destname(), "System error: (%1) %2", cache_entry->m_cache_info.m_errno, strerror(cache_entry->m_cache_info.m_errno));
+                Logging::error(cache_entry->virtname(), "System error: (%1) %2", cache_entry->m_cache_info.m_errno, strerror(cache_entry->m_cache_info.m_errno));
             }
             if (cache_entry->m_cache_info.m_averror)
             {
-                Logging::error(cache_entry->destname(), "FFMpeg error: (%1) %2", cache_entry->m_cache_info.m_averror, ffmpeg_geterror(cache_entry->m_cache_info.m_averror).c_str());
+                Logging::error(cache_entry->virtname(), "FFMpeg error: (%1) %2", cache_entry->m_cache_info.m_averror, ffmpeg_geterror(cache_entry->m_cache_info.m_averror).c_str());
             }
         }
     }
@@ -1012,17 +1030,132 @@ static void transcoder_thread(void *arg)
     delete thread_data;
 
     errno = syserror;
+
+    return errno;
 }
 
 void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
 {
+    Logging::LOGLEVEL ffmpegfs_level;
+
+    // Map log level
+    // AV_LOG_PANIC     0
+    // AV_LOG_FATAL     8
+    // AV_LOG_ERROR    16
+    if (level <= AV_LOG_ERROR)
+    {
+        ffmpegfs_level = LOGERROR;
+    }
+    // AV_LOG_WARNING  24
+    else if (level <= AV_LOG_WARNING)
+    {
+        ffmpegfs_level = LOGWARN;
+    }
+#ifdef AV_LOG_TRACE
+    // AV_LOG_INFO     32
+    //else if (level <= AV_LOG_INFO)
+    //{
+    //    ffmpegfs_level = LOGINFO;
+    //}
+    // AV_LOG_VERBOSE  40
+    // AV_LOG_DEBUG    48
+    else if (level < AV_LOG_DEBUG)
+    {
+        ffmpegfs_level = LOGDEBUG;
+    }
+    // AV_LOG_TRACE    56
+    else   // if (level <= AV_LOG_TRACE)
+    {
+        ffmpegfs_level = LOGTRACE;
+    }
+#else
+    // AV_LOG_INFO     32
+    else   // if (level <= AV_LOG_INFO)
+    {
+        ffmpegfs_level = DEBUG;
+    }
+#endif
+
+    if (!Logging::show(ffmpegfs_level))
+    {
+        return;
+    }
+
     va_list vl2;
-    Logging::level ffmpegfs_level;
     static int print_prefix = 1;
 
 #if (LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 23, 0))
     char * line;
     int line_size;
+    std::string category;
+
+    if (ptr != nullptr)
+    {
+        AVClass* avc = *(AVClass **)ptr;
+
+        switch (avc->category)
+        {
+        case AV_CLASS_CATEGORY_NA:
+        {
+            break;
+        }
+        case AV_CLASS_CATEGORY_INPUT:
+        {
+            category = "INPUT   ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_OUTPUT:
+        {
+            category = "OUTPUT  ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_MUXER:
+        {
+            category = "MUXER   ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_DEMUXER:
+        {
+            category = "DEMUXER ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_ENCODER:
+        {
+            category = "ENCODER ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_DECODER:
+        {
+            category = "DECODER ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_FILTER:
+        {
+            category = "FILTER  ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_BITSTREAM_FILTER:
+        {
+            category = "BITFILT ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_SWSCALER:
+        {
+            category = "SWSCALE ";
+            break;
+        }
+        case AV_CLASS_CATEGORY_SWRESAMPLER:
+        {
+            category = "SWRESAM ";
+            break;
+        }
+        default:
+        {
+            category = strsprintf("CAT %3i ", static_cast<int>(avc->category));
+            break;
+        }
+        }
+    }
 
     va_copy(vl2, vl);
     av_log_default_callback(ptr, level, fmt, vl);
@@ -1048,45 +1181,7 @@ void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
     va_end(vl2);
 #endif
 
-    // Map log level
-    // AV_LOG_PANIC     0
-    // AV_LOG_FATAL     8
-    // AV_LOG_ERROR    16
-    if (level <= AV_LOG_ERROR)
-    {
-        ffmpegfs_level = LOGERROR;
-    }
-    // AV_LOG_WARNING  24
-    else if (level <= AV_LOG_WARNING)
-    {
-        ffmpegfs_level = LOGWARN;
-    }
-#ifdef AV_LOG_TRACE
-    // AV_LOG_INFO     32
-    else if (level <= AV_LOG_INFO)
-    {
-        ffmpegfs_level = LOGINFO;
-    }
-    // AV_LOG_VERBOSE  40
-    // AV_LOG_DEBUG    48
-    else if (level <= AV_LOG_DEBUG)
-    {
-        ffmpegfs_level = LOGDEBUG;
-    }
-    // AV_LOG_TRACE    56
-    else   // if (level <= AV_LOG_TRACE)
-    {
-        ffmpegfs_level = LOGTRACE;
-    }
-#else
-    // AV_LOG_INFO     32
-    else   // if (level <= AV_LOG_INFO)
-    {
-        ffmpegfs_level = DEBUG;
-    }
-#endif
-
-    Logging::log_with_level(ffmpegfs_level, "", line);
+    Logging::log_with_level(ffmpegfs_level, "", category + line);
 
 #if (LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 23, 0))
     av_free(line);
@@ -1095,7 +1190,7 @@ void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
 
 bool init_logging(const std::string &logfile, const std::string & max_level, bool to_stderr, bool to_syslog)
 {
-    static const std::map<const std::string, const Logging::level, comp> level_map =
+    static const std::map<const std::string, const Logging::LOGLEVEL, comp> level_map =
     {
         { "ERROR",      LOGERROR },
         { "WARNING",    LOGWARN },
@@ -1104,7 +1199,7 @@ bool init_logging(const std::string &logfile, const std::string & max_level, boo
         { "TRACE",      LOGTRACE },
     };
 
-    std::map<const std::string, const Logging::level, comp>::const_iterator it = level_map.find(max_level);
+    std::map<const std::string, const Logging::LOGLEVEL, comp>::const_iterator it = level_map.find(max_level);
 
     if (it == level_map.cend())
     {
